/** author: Tim vor der Brck
 * this software is released under the GNU General Public License
 */

#ifndef PATH_H
#define PATH_H

#include <string>
#include <iostream>
#include <fstream>
#include "StringParser.h"

const char PATH_SEPARATOR=':';
const char FILE_SEPARATOR='/';

/** removes all unncecessary '/' at the end of a path*/
string simplifyPath(string path,char fileSep)
{
  int i=path.length()-1;
  while (i>0 && path[i]==fileSep)
    i--;
  return path.substr(0,i+1);
}

string _getParent(string path,char fileSep)
{
  char str[]={fileSep,0};
  string root=str,empty="";
  if (path==root) return empty;
  int i=path.find_last_of(fileSep);
  if (i==-1) return empty;
  return path.substr(0,i);
}

/** get directory that contains file*/
string getParent(string path,char fileSep)
{
  path=simplifyPath(path,fileSep);
  return _getParent(path,fileSep);
}

/** returns true if a file exists*/
bool exists(string filename)
{
  ifstream f(filename.c_str());
  if (f==0) return false;
  return true;
}



/** get the directory the application is located in
 * @param  args[0]
 * @return  program directory or "" if not found
 */
string getPrgDir(string filename,char fileSep,char pathSep)
{
  //filename contains /: no need to search path list
   if (filename.find(fileSep)!=-1)
  {
    if (exists(filename)) return getParent(filename,fileSep);
    return string("");
  }
   int i1=-1,i2;

  //fullPath=pathName+fileSep+filename
  string pathName,fullPath,pathList=getenv("PATH");
  string delims[]={string(1,pathSep)};
  StringParser parser(pathList,delims,1);
  StringParser::const_iterator iter=parser.begin();
 
  while (iter!=parser.end())
  {
    pathName=*iter;
    fullPath=pathName+fileSep+filename;
    if (exists(fullPath)) return pathName;
    ++iter;
  }
  return string("");
}
 
string getPrgDir(string filename)
{
  return getPrgDir(filename,FILE_SEPARATOR,PATH_SEPARATOR);
}


#endif 



