/*Copyright: Tim vor der Brck 1999

This software is released under the GNU General Public License.
It may be redistributed as long this copyright remark is not
removed or modified.
*/

#include <algorithm>
#include "icon.h"

using namespace std;

/*int min(int x,int y)
{
  if (x<y) return x;
  return y;
}

int max(int x,int y)
{
  if (x>y) return x;
  return y;
}*/

//******************************* create ****************************
 /** 
  * creates an icon 
  *
  * @param pdisplay  default display 
  * @param p  Icon pixmap
    @param mask  mask pixmap of the outline of the pixmap
    @param width  width of the icon
    @param height  heiht of the icon
  */ 
void CIcon::create(Display* pdisplay,Pixmap p,Pixmap mask,GC gc,int width,int height)
{
  m_width=width;
  m_height=height;
  m_gc=gc;
  m_mask=mask;
  m_pixmap=p;
  m_pdisplay=pdisplay;
}

//***************************** draw ************************************
 /** 
  * draws the icon an position x,y at pixmap dest 
  *
  * @param x  x coordinate the icon will be drawn
  * @param y  y coordinate the icon will be drawn
  * @param dest  drawable the icon will be drawn 
  */
void CIcon::draw(int x,int y,Drawable dest)
{
  XSetFunction(m_pdisplay,m_gc,GXand);
  XCopyArea(m_pdisplay,m_mask,dest,m_gc,0,0,m_width,m_height,x,y);
  XSetFunction(m_pdisplay,m_gc,GXor);
  XCopyArea(m_pdisplay,m_pixmap,dest,m_gc,0,0,m_width,m_height,x,y);
  XSetFunction(m_pdisplay,m_gc,GXcopy);
}

//**************************** remove ******************************
/** 
  * removes the icon from the drawable
  * 
  * @param x  x coordinate of the icon on the drawable
  * @param y  y coordinate of the icon on the drawable
  * @param source  background pixmap
  * @param dest  drawable that will be drawon onto 
  */
void CIcon::remove(int x,int y,Drawable source,Drawable dest)
{
  XCopyArea(m_pdisplay,source,dest,m_gc,x,y,m_width,m_height,x,y);
}

//***************************** begindrag *******************************
 /**
  * begins a drag operation of this icon
  *
  * @param x  x coordintes of the position the icon will be drawn
  * @param y  y coordinate 
  * @param source  background pixmap
  * @param dest  drawable where the icon will be drawn, because of performance
  *              reasons this should not be a window but another pixmap,
  *              to make the change visible on the screen use dbdraw
  */
void CIcon::begindrag(int x,int y,Drawable source,Drawable dest)
{
  m_dndsource=source;
  m_dnddest=dest;
  draw(x,y,dest);
  m_dndoldx=x;
  m_dndoldy=y;
  m_dndx0=x;
  m_dndy0=y;
  m_dndx1=x+m_width;
  m_dndy1=y+m_width;
}

//****************************** dodrag ***********************************
 /**
  * should be called if you move the pointer with pressed mouse button
  *  
  * @param x  x coordinate where the icon will be drawn
  * @param y  y coordinate
  */   
void CIcon::dodrag(int x,int y)
{
  //remove old
  remove(m_dndoldx,m_dndoldy,m_dndsource,m_dnddest);

  //draw new
  draw(x,y,m_dnddest);

 
  m_dndx0=min(x,m_dndoldx);
  m_dndy0=min(y,m_dndoldy);
  m_dndx1=max(x+m_width,m_dndoldx+m_width);
  m_dndy1=max(y+m_height,m_dndoldy+m_height);

 //save old values
  m_dndoldx=x;
  m_dndoldy=y;
} 

//****************************** drop *************************************
 /**
  * ends a drag operation 
  */
void CIcon::drop()
{
  remove(m_dndoldx,m_dndoldy,m_dndsource,m_dnddest);

  m_dndx0=m_dndoldx;
  m_dndy0=m_dndoldy;
  m_dndx1=m_dndoldx+m_width;
  m_dndy1=m_dndoldy+m_height;
}

//****************************** dbdraw ***********************************
 /**
  * should be called after begindrag,dodrag and drop
  * makes the drag operation visible on the screen
  * 
  * @param source  souce pixmap, this should be the destination pixmap of begindrag
  * @param dest  this should be the window
  */
void CIcon::dbdraw(Pixmap source,Pixmap dest)
{
    XCopyArea(m_pdisplay,source,dest,m_gc,m_dndx0,m_dndy0,
          m_dndx1-m_dndx0,m_dndy1-m_dndy0,m_dndx0,m_dndy0);
 
}




