/*Copyright: Tim vor der Brck 1999

This software is released under the GNU General Public License.
It may be redistributed as long as this copyright remark is not
removed or modified.
*/


#ifndef CIconh
#define CIconh

//**************************** CIcon *****************************
/**
 * icon class that supports transparency and drag and drop
 * icons can be of any size 
 */

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>

class CIcon
{
 public:

  //******************************* create ****************************
  /** 
   * creates an icon 
   *
   * @param pdisplay  default display 
   * @param p  Icon pixmap
     @param mask  mask pixmap of the outline of the pixmap
     @param width  width of the icon
     @param height  heiht of the icon
   */ 
  void create(Display* pdisplay,Pixmap p,Pixmap mask,GC gc,int width,int height);

  //***************************** draw ************************************
  /** 
   * draws the icon an position x,y at pixmap dest 
   *
   * @param x  x coordinate the icon will be drawn
   * @param y  y coordinate the icon will be drawn
   * @param dest  drawable the icon will be drawn 
   */
  void draw(int x,int y,Drawable dest);

  //**************************** remove ******************************
  /** 
   * removes the icon from the drawable
   * 
   * @param x  x coordinate of the icon on the drawable
   * @param y  y coordinate of the icon on the drawable
   * @param source  background pixmap
   * @param dest  drawable that will be drawon onto 
   */
  void remove(int x,int y,Drawable source,Drawable dest);
  
  //***************************** begindrag *******************************
  /**
   * begins a drag operation of this icon
   *
   * @param x  x coordintes of the position the icon will be drawn
   * @param y  y coordinate 
   * @param source  background pixmap
   * @param dest  drawable where the icon will be drawn, because of performance
   *              reasons this should not be a window but another pixmap,
   *              to make the change visible on the screen use dbdraw
   */
  void begindrag(int x,int y,Drawable source,Drawable dest);
  
  //****************************** dodrag ***********************************
  /**
   * should be called if you move the pointer with pressed mouse button
   *  
   * @param x  x coordinate where the icon will be drawn
   * @param y  y coordinate
   */   
  void dodrag(int x,int y);

   //****************************** drop *************************************
  /**
   * ends a drag operation 
   */
  void drop();

   //****************************** dbdraw ***********************************
  /**
   * should be called after begindrag,dodrag and drop
   * makes the drag operation visible on the screen
   * 
   * @param source  souce pixmap, this should be the destination pixmap of begindrag
   * @param dest  this should be the window
   */
  void dbdraw(Pixmap source,Pixmap dest);

  //****************************** getwidth *********************************
  /**
   * returns the width of the icon
   * 
   * @return width  witdh of the icon
   */ 
  int getwidth(){return m_width;}
 
  //****************************** getheight *********************************
  /**
   * return the height of the icon
   *
   * @return  the height of the icon
   */
  int getheight(){return m_height;}
 protected:
  int m_dndx0;
  int m_dndy0;
  int m_dndx1;
  int m_dndy1;
  int m_dndoldx;
  int m_dndoldy;
  Pixmap m_dndsource;
  Pixmap m_dnddest;
  int m_width;
  int m_height;
  GC m_gc;
  Pixmap m_pixmap;
  Pixmap m_mask;
  Display* m_pdisplay;
  
};
  

#endif
