#include <algorithm>
#ifdef HAVE_STRSTREAM
#include <strstream.h>
#else
#include <sstream>
#endif
#include <iostream>
#include <string.h>
#include <string>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/xpm.h>
#include <vbase/error.h>
extern "C" 
{
#include <vwidgets/IconSel.h>
}
#include "Cjchess.h"
#include "jchess.h"

using namespace std;

char* miconfilenames[]={"rock.Xpm","janus.Xpm",
			"knight.Xpm","bishop.Xpm",
			"queen.Xpm", "king.Xpm",
			"pawn.Xpm"};

extern bool bsetuppos;
extern string prgDir;

//********************* Jchess ********************
Cjchess::Cjchess()
{
  m_gcicons=0;
  m_sp=0;
  m_level[0]=0;
  m_level[1]=5;
  m_gamestate=normal;
  m_bcomputermove=false;
  m_bsetuppos=false;
  m_win=0;
  m_output=0;
  m_input=0;
  m_psd=NULL;
  m_lastmove=leererzug;
}

//********************* ~Cjchess ********************
Cjchess::~Cjchess()
{
  if (m_gcicons!=0) XFreeGC(m_display,m_gcicons);
  if (m_output!=0) fclose(m_output);
  if (m_input!=0) fclose(m_input);
  if (m_psd!=NULL) delete m_psd;
}

//********************* create ********************
/**
 * @param board  logical board
 * @param boardview  graphical view of the board
 * @param display  display used
 * @param dx  width of the board
 * @param dy  height of the board
 * @param sd  shawhite data with second thread
 * @param pappres  application resource data
 * @param statuswindow  statuswindow with information about last move, state of the game ...
 * @param takeback  menu widget to take back last move
 * @param lastown  menu widget to convert to last own move
 */
void Cjchess::create(jsbrettklasse* board,CBoardview* boardview,
                     Display* display,
                     int dx,int dy,SharedData* sd,
                     ResData* pappres,Widget statuswindow,
                     Widget takeback,Widget lastown)
{
  // set up variables
  m_pboard=board;
  m_pboardview=boardview;
  m_display=display;
  m_psd=sd;
  m_pappres=pappres;
  m_statuswindow=statuswindow;
  m_takeback=takeback;
  m_lastown=lastown;
  //m_win=win;
  
  m_output=fdopen((sd->pipe_tothread)[1],"w");
  m_input=fdopen((sd->pipe_toapp)[0],"r");

  // create graphics context for loading icons
  XGCValues gcvalues;
  m_screennr=DefaultScreen(display);
  int depth=DefaultDepth(display,m_screennr);
  int fg=BlackPixel(display,m_screennr);
  int bg=WhitePixel(display,m_screennr);
  gcvalues.foreground=0; //fg;
  gcvalues.background=(1<<depth)-1; //(int)pow(2,depth)-1+0.5; //255; //bg;
  m_rootwin=RootWindow(m_display,m_screennr);
  m_gcicons=XCreateGC(m_display,m_rootwin,GCForeground|GCBackground,&gcvalues);
 
  // load icons
  loadicons(m_pappres->scolorwhite,m_pappres->scolorblack);

  // init logical board
  m_pboard->setzelevel(10);
  m_pboard->init();
  
  //init graphical board
  m_pboardview->create(this,board,m_display,0,dx,dy,xmax+3,ymax+3);  

  update(false,false);
}


//********************* drawboard ********************
/**
 * draws the empty board in pixmap pixmap
 * will be called by the boardview class
 * @param display  display used 
 * @param pixmap  destination pixmap
 * @param dx  width of the board
 * @param dy  height of the board
 */
void Cjchess::drawboard(Display* display,GC gcbg,Pixmap& pixmap,int dx,int dy)
{
  Colormap colormap;
  int fdx=dx/(xmax+3),fdy=dy/(ymax+3),x,y,winx,winy; //11,12
  //Gitter
  //vertikal
  //XClearArea(display,pixmap,0,0,200,200,FALSE);
  int screennr=DefaultScreen(display);
  //int fg=BlackPixel(display,screennr);
  //int bg=WhitePixel(display,screennr);
  XColor cobgboard,colorex,coblackfield,cowhitefield;

  // clear pixmap
  colormap=DefaultColormap(display,screennr);
  XAllocNamedColor(display,colormap,m_pappres->sbackgroundboard,&cobgboard,&colorex);
  XAllocNamedColor(display,colormap,m_pappres->sblackfieldcolor,&coblackfield,&colorex);
  XAllocNamedColor(display,colormap,m_pappres->swhitefieldcolor,&cowhitefield,&colorex);
  XSetForeground(display,gcbg,cobgboard.pixel);
  XFillRectangle(display,pixmap,gcbg,0,0,dx,dy);
  //XSetForeground(display,gcbg,fg);

  for (x=1,winx=fdx;x<=10;x=x+1,winx=winx+fdx)
  {
    for (y=1,winy=fdy;y<=8;y++,winy+=fdy)
    {
      if ((x+y)%2==0)
        XSetForeground(display,gcbg,coblackfield.pixel);
      else
	XSetForeground(display,gcbg,cowhitefield.pixel);
      XFillRectangle(display,pixmap,gcbg,winx,winy,fdx,fdy);
    }
  }
	  
  // draws letters and numbers around the board
  drawcoords(display,gcbg,pixmap,fdx,fdy,dy);
}  

//********************* convvaluetoicon ********************
/**
 * returns the icon of a piece out of the logical value
 * is called by the class boardview
 *
 * @param value  logical value 
 *
 * @return  corresponding icon
 */
CIcon* Cjchess::convvaluetoicon(int value)
{
  if (value<0) return &(m_icons[1][-value-1]);
  else return &(m_icons[0][value-1]);
} 


//********************* loadicons ********************
void Cjchess::loadicons(char* white,char* black)
{
  string iconFilename;
  Pixmap iconpixmap,mask,maskbm;
  XpmAttributes attr;
  Window defrootwin=DefaultRootWindow(m_display);
  int status,depth=DefaultDepth(m_display,m_screennr);
  XpmColorSymbol symbol[2];
  attr.valuemask=XpmColorSymbols;
  attr.colorsymbols=symbol;
  attr.numsymbols=2;
  symbol[0].name="Background";
  symbol[1].name="Foreground";
    

  for (int i=0;i<numberoficons;i++)
  {
    // white
    symbol[0].value=white;
    symbol[1].value=black;

    // load pixmap
    iconFilename=prgDir+string(miconfilenames[i]);
    status=XpmReadFileToPixmap(m_display,m_rootwin,(char*)iconFilename.c_str(),&iconpixmap,&(m_masks[i]),&attr);
    if (status!=XpmSuccess) displayfileerror(miconfilenames[i],"loadicon",true);

    // create mask for icon
    mask=XCreatePixmap(m_display,defrootwin,attr.width,attr.height,depth);
    XCopyPlane(m_display,m_masks[i],mask,m_gcicons,0,0,attr.width,attr.height,0,0,1);     
     
    // create icon with this pixmap
    m_icons[0][i].create(m_display,iconpixmap,mask,m_gcicons,attr.width,attr.height);
    
    // black
    symbol[0].value=black;
    symbol[1].value=white;
    iconFilename=prgDir+string(miconfilenames[i]);
    // load two times once for white and once for black
    status=XpmReadFileToPixmap(m_display,m_rootwin,(char*)iconFilename.c_str(),&iconpixmap,NULL,&attr);
    if (status!=XpmSuccess) displayfileerror((char*)iconFilename.c_str(),"loadicon",true);
    m_icons[1][i].create(m_display,iconpixmap,mask,m_gcicons,attr.width,attr.height);
  }
}

//********************* move ********************
/**
 * moves a piece from one field to another
 * @param x1  x-coordinate of source field (0 is one field outside the board)
 * @param y1  y-coordinate of source field
 * @param x2  x-coordinate of destination field
 * @param y2  y-coordinate of destination field
 * @param remove  if true, removes the icon at the source field, not necessary
 *                with drag and drop because icon gets deleted automatically
 */ 
void Cjchess::smove(zug z,bool remove)
{  
  //logical move
  m_pboard->ziehereg(z);

  //graphical move
  CCounter* ss;
  if (z.ep==0 && z.bef==0 && (abs(z.x2-z.x1)==1 || abs(m_pboard->getcontents(z.x2,z.y2))!=koenig))
  {
    ss=m_pboardview->getcontents(z.x1+1,z.y1+1);
    ss->smove(z.x2+1,z.y2+1,remove);
  }
  else 
  {
    m_pboardview->update();
    m_pboardview->draw();
  }
  // analyse gamesituation
  m_gamestate=m_pboard->analyse(m_sp);

  m_sp=!m_sp;
  m_lastmove=z;
  updatestatuswindow();
  updatemenus();
}

//********************* start_computermove ********************
void Cjchess::start_computermove()
{
  m_bcomputermove=true;
  m_psd->command=0;
  sendmessage(startcalcmove);
}

//********************* sendmessage ********************
void Cjchess::sendmessage(char c)
{
  fputc(c,m_output);
  fflush(m_output);
}

//********************* do_computermove ********************
void Cjchess::do_computermove()
{
  m_bcomputermove=false;
  zug bestmove=m_psd->bestmove;
  if (bestmove==leererzug) cout << "Could not find any move!" << endl;
  if (m_psd->command>=100 || bestmove==leererzug) return;
  smove(bestmove,true);
  if ((m_gamestate==normal || m_gamestate==istschach) && m_level[m_sp]>0) start_computermove();
}

//*************************** update ********************************
void Cjchess::update(bool banalyse,bool bdraw)
{
  m_pboardview->update();
  if (banalyse) m_gamestate=m_pboard->analyse(m_sp);  
  updatestatuswindow();
  if (bdraw)
  {
     m_pboardview->draw();
     //XtRealizeWidget(m_statuswindow);
  }
  updatemenus();    
}

//**************************** newgame ********************************
void Cjchess::newgame()
{
  abort_computermove();
  m_sp=0;
  m_pboard->init();
  update();
  if (m_level[0]>0) start_computermove();
}

//*************************** abortcomputermove **********************************
void Cjchess::abort_computermove()
{
  if (m_bcomputermove) 
  {
    m_psd->command=100; //abort command
    char c=fgetc(m_input); //wait for abortion
  }
  m_bcomputermove=false;
}

//**************************** createstatusoutput ******************************
void Cjchess::createstatusoutput()
{
#ifdef HAVE_STRSTREAM
  ostrstream s(m_statusoutput,100);
#else
  ostringstream s;
#endif

  // 1. Zeile Moving: White
  s << m_pappres->smoving << ": ";
  if (m_sp==0) s << m_pappres->swhite;
  else s << m_pappres->sblack;

  //2. row: last move
  s << endl << m_pappres->slastmove << ": " << m_lastmove << endl;

  //3. row:
  s << m_pappres->slevel << ": ";
  int lev0=m_level[0],lev1=m_level[1];
  if (lev0!=0)
  {
    if (lev1!=0)
      s << lev0 << ", "<< lev1;
    else 
      s << lev1;
  }
  else if (lev1!=0) s << lev1;
  s << endl;
  
  //state of game
  s << m_pappres->sstate << ": ";
  switch (m_gamestate)
  {
    case normal: s << m_pappres->snormal;break;
    case istschach: s << m_pappres->scheck;break;
    case matt: s << m_pappres->smate;break;
    case patt: s << m_pappres->sstalemate;break;
    case istremis: s << m_pappres->sremis;
  }   
  s << endl;

  // and the winner is
  if (m_gamestate==matt)
  {
    if (m_sp==1) s << m_pappres->swhite << ' ' << m_pappres->swins << endl;
    else s << m_pappres->sblack << ' ' << m_pappres->swins << endl;
  }
  s << '\0';
#ifndef HAVE_STRSTREAM
  strcpy(m_statusoutput,s.str().c_str());
#endif
}

//*********************** updatestatuswindow ************************
void Cjchess::updatestatuswindow()
{
  createstatusoutput();
  XtVaSetValues(m_statuswindow,XtNlabel,m_statusoutput,NULL);
}


//************************ oncancel *****************************
/* will be called when you cancel the playersetupdialog */
void Cjchess::oncancel()
{
  popdownplayersetupdialog();
  if (m_level[m_sp]>0 && (m_gamestate==normal || m_gamestate==istschach)) start_computermove();
}

//************************* setvalues *******************************
/* will be called when you choose ok in the playersetupdialog  */
void Cjchess::setvalues(int lev0,int lev1) //,Boolean player1ishuman,Boolean player2ishuman)
{
  bool ret;

  ret=m_pboard->setzelevel(max(lev0,lev1));
  if (!ret)
     popupMessageBox("Error","Out of memory error!");
  else
  {
    popdownplayersetupdialog();
    m_level[0]=lev0;
    m_level[1]=lev1;
    updatemenus();
    if (m_level[m_sp]>0 && (m_gamestate==normal || m_gamestate==istschach))
	start_computermove();
   }
}

// ***************** takebackmove *****************
void Cjchess::takebackmove(bool upd)
{      
  
        // get last move 
	int hz=m_pboard->hz; 
        if (hz<1) return; // not possible to take back last move because it is the first!
	zug z=m_pboard->holezug(hz-1); //letzter Zug

	//error?
	if (z==leererzug) return;
	
	//take back move logically
	stein s=m_pboard->zuruecknehmenreg();

	//take back move graphically
	CCounter* ss;
        if (z.ep==0 && z.bef==0 && (abs(z.x2-z.x1)==1 || abs(m_pboard->getcontents(z.x1,z.y1))!=koenig))
	{
	  ss=m_pboardview->getcontents(z.x2+1,z.y2+1); 
	  ss->smove(z.x1+1,z.y1+1);
	  if (s!=0) m_pboardview->setcontents(z.x2+1,z.y2+1,  //a piece was stroke?
		new CCounter(z.x2+1,z.y2+1,convvaluetoicon(s),
			s,m_pboardview));
	}
        else
	{
	  m_pboardview->update();
	  m_pboardview->draw();
	}
	
        // other player is moving now
        m_sp=!m_sp;
        m_gamestate=m_pboard->analyse(m_sp);
	if (upd)
        {
          m_gamestate=m_pboard->analyse(m_sp);
          m_lastmove=m_pboard->holezug(m_pboard->hz-1);
	  updatestatuswindow();
        }
}



ostream& operator<<(ostream& os,const Cjchess& cs)
{
  os << "jchess"  << endl;
  os << cs.m_level[0] << ' ' << cs.m_level[1] << endl;
  os << *(cs.m_pboard);
  return os;
}

istream& operator>>(istream& is,Cjchess& cs)
{
  char str[256];
  is >> str;
  if (strcmp(str,"jchess")!=0) throw "Invalid file format"; 
  is >> cs.m_level[0];
  is >> cs.m_level[1];
  is >> *(cs.m_pboard);
  cs.m_sp=cs.m_pboard->hz%2;
  return is;
}
  
//******************************* drawcoords ***********************************
void Cjchess::drawcoords(Display* d,GC gc,Pixmap& pixmap,int fdx,int fdy,int dy)
{
  XFontStruct* pfs;
  char strlet[]="A"; //letters
  char strnum[3]; //numbers
  int num=1;
  pfs=XLoadQueryFont(d,"*courier-*-*-18*");
  if (pfs==0) 
  {
    cout << "unable to load font" << endl;
    return;
  }
  XSetFont(d,gc,pfs->fid);  

  //display letters
  for (int x=1;x<=xmax+1;x++,strlet[0]++)
    drawstring(d,gc,pixmap,strlet,x,0,fdx,fdy,dy,pfs);

  //display number
  for (int y=1;y<=ymax+1;y++,num++)
  {
    gcvt(num,2,strnum);
    drawstring(d,gc,pixmap,strnum,0,y,fdx,fdy,dy,pfs);
  }     
}

//******************************* drawstring ********************************** 
void Cjchess::drawstring(Display* d,GC gc,Pixmap& pixmap,char* str,int x,int y,
        int fdx,int fdy,int dy,XFontStruct* fs)
{
  int winx=x*fdx+(fdx-XTextWidth(fs,str,strlen(str)))/2;
  int winy=dy-y*fdy-(fdy-fs->ascent-fs->descent)/2;
  XDrawString(d,pixmap,gc,winx,winy,str,strlen(str));
}


//***************************** updatemenus **********************************
/**
 * make some menus appear in grey that they cannot be selected
 */
void Cjchess::updatemenus()
{
  if (m_pboard->hz>0) XtVaSetValues(m_takeback,XtNsensitive,TRUE,NULL);
  else XtVaSetValues(m_takeback,XtNsensitive,FALSE,NULL);
  if ((m_pboard->hz>0 && m_level[0]==0) ^ (m_pboard->hz>1 && m_level[1]==0))
    XtVaSetValues(m_lastown,XtNsensitive,TRUE,NULL);
  else XtVaSetValues(m_lastown,XtNsensitive,FALSE,NULL);
}

void Cjchess::psdhandleerror(int errnum)
{
  switch (errnum)
  {
      case PSDOUTOFRANGE: 
	popupMessageBox(m_pappres->serror,m_pappres->soutofrange);
        return;
      case PSDNOSELECT:
        popupMessageBox(m_pappres->serror,m_pappres->snoselected);
        return;
      default:
        cerr << "Invalid error code!" << endl;
  }
} 

bool Cjchess::exists(int value,int f[xmax+1][ymax+1],int startx,int endx,int starty,int endy)
{
  for (int x=startx;x<=endx;x++)
    for (int y=starty;y<=endy;y++)
      if (f[x][y]==value) return true;
  return false;
}

bool Cjchess::valid(int f[xmax+1][ymax+1])
{
  if (!(exists(koenig,f,3,5,0,2)) || !(exists(-koenig,f,3,5,7,9))) return false;
  return true;
}

typedef struct _iconseldata
{
  GC gc;
  Display* display;
  Widget iconselpopup;
  Widget iconsel;
  Pixmap* bitmaps;
  zug z;
  Cjchess* pboardgame;
} iconseldata;


void oniconselect(Widget w,XtPointer client,XtPointer call)
{
  
  //close dialog
  iconseldata* picsel=(iconseldata*)client;
  XtDestroyWidget(picsel->iconselpopup);
  XFreeGC(picsel->display,picsel->gc);

  // deallocate 
  /*  for (int i=0;i<5;i++)
      XFreePixmap(picsel->display,picsel->bitmaps[i]);*/
  

  //move
  
  zug z=picsel->z;
  SelectInfo* pselinfo=(SelectInfo*)call;
  z.bef=pselinfo->no+1;
  if (z.y2==0) z.bef=-z.bef; //black moved
  Cjchess* pboardgame=picsel->pboardgame;
  pboardgame->smove(z,false); //move without erase source field, anyway erased
  
  delete picsel;
}


void Cjchess::popupiconsel(Widget toplevel,zug& z)
{
  int i,j;

  //alloc struct for storing icon selector data
  iconseldata* picsel=new iconseldata;
  picsel->z=z;  
    
  //alloc GC
  picsel->display=XtDisplay(toplevel);
  picsel->gc=XCreateGC(picsel->display,m_rootwin,0,NULL);

  picsel->pboardgame=this;
  
  //init bitmaps
  Window defrootwin=DefaultRootWindow(picsel->display);
  CIcon* picon=convvaluetoicon(1);
  int width=32; //this would create segmentation faultpicon->getwidth();
  int height=32; //picon->getheight();
  
  picsel->bitmaps=m_masks;

  picsel->bitmaps[5]=0;

  picsel->iconselpopup=XtVaCreatePopupShell("iconselpopup",transientShellWidgetClass,toplevel,NULL);
  picsel->iconsel=XtVaCreateManagedWidget("iconsel",iconSelectorWidgetClass,picsel->iconselpopup,
					  XvNbitmaps,picsel->bitmaps,NULL);
  XtAddCallback(picsel->iconsel,XtNcallback,oniconselect,picsel);
  XtPopup(picsel->iconselpopup,XtGrabNonexclusive);

}


