/* file jchess.cpp
    Author: Tim vor der Brueck
    Version 0.9
*/

//******************************* includes **************************************

#ifdef suse
#include <pthread/mit/pthread.h>
#else
#include <pthread.h>
#endif
#include <sys/types.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fstream>
#include <sstream>
#ifdef HAVE_STRSTREAM
#include <strstream>
#else
#include <sstream>
#endif
#include <string>
#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/SimpleMenu.h>
#include <X11/Xaw/SmeBSB.h>
#include <X11/Xaw/SmeLine.h>
#include <X11/Xaw/MenuButton.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Command.h>
#include <X11/xpm.h>
#include <vgui/menu.h>
#include <vbase/path.h>
#include "Cjchess.h"
#include "jchess.h"
#include "jsbrett.h"

extern "C"
{
#include <FileNom.h>
}

using namespace std;

//*********************** global Variables *****************
jsbrettklasse board;
CBoardview boardview;
Cjchess cs; 
Display* display;
bool bfirsttime=true,bdrag=false,bfirstconf=true,bmove=true,bload,
     bsetuppos=false;
int xx1,yy1,xx2,yy2;
XtAppContext app_context;
Widget toplevel,fileselpopup=0,filesel=0,setuppos,ocpopup=0,ocform,oclabel,ocok,occancel,
   form,wbox,button_partie,game_new,game_quit,menu_partie,statuswindow,label,players,
   game_load,game_save,game_saveas,button_move,menu_move,takeback,lastown,button_board,
   menu_board,clearboard,help;
char game_name[512]="",game_nameoverwrite[512];
int f[xmax+1][ymax+1];
ResData res;
string prgDir;

void onoverwriteok(Widget w,XtPointer client,XtPointer call);
void onoverwritecancel(Widget w,XtPointer client,XtPointer call);
void loadorsave(Widget w,XtPointer client,XtPointer call);
void oncancelfiledialog(Widget w,XtPointer client,XtPointer call);

static XtResource resources[]=
{  
  {"swhite","SWhite",XtRString,sizeof(String),XtOffsetOf(ResData,swhite),XtRString,String("White")},
  {"sblack","SBlack",XtRString,sizeof(String),XtOffsetOf(ResData,sblack),XtRString,String("Black")},
  {"slevel","SLevel",XtRString,sizeof(String),XtOffsetOf(ResData,slevel),XtRString,String("Level")},
  {"mate","Mate",XtRString,sizeof(String),XtOffsetOf(ResData,smate),XtRString,String("Checkmate")},
  {"statemate","Stalemate",XtRString,sizeof(String),XtOffsetOf(ResData,sstalemate),XtRString,String("Stalemate")},
  {"normal","Normal",XtRString,sizeof(String),XtOffsetOf(ResData,snormal),XtRString,String("Normal")},
  {"remis","Remis",XtRString,sizeof(String),XtOffsetOf(ResData,sremis),XtRString,String("Remis")},
  {"check","Check",XtRString,sizeof(String),XtOffsetOf(ResData,scheck),XtRString,String("Check")},
  {"wins","Wins",XtRString,sizeof(String),XtOffsetOf(ResData,swins),XtRString,String("wins")},
  {"moving","Moving",XtRString,sizeof(String),XtOffsetOf(ResData,smoving),XtRString,String("Moving")},
  {"lastmove","LastMove",XtRString,sizeof(String),XtOffsetOf(ResData,slastmove),XtRString,String("Last Move")},
  {"state","State",XtRString,sizeof(String),XtOffsetOf(ResData,sstate),XtRString,String("State")},
  {"helpapp","Helpapp",XtRString,sizeof(String),XtOffsetOf(ResData,shelpapp),XtRString,String("netscape")},
  {"helpfile","Helpfile",XtRString,sizeof(String),XtOffsetOf(ResData,shelpfile),XtRString,String("docs/jchess.html")},
  {"serror","SError",XtRString,sizeof(String),
   XtOffsetOf(ResData,serror),XtRString,String("Error")},
  {"invbo","Invbo",XtRString,sizeof(String),XtOffsetOf(ResData,sinvbo),XtRString,String("Invalid board!")},
  {"outofrange","Outofrange",XtRString,sizeof(String),XtOffsetOf(ResData,
     soutofrange),XtRString,String("Level must be a number from 1 till 99")},
  {"noselected","Noselected",XtRString,sizeof(String),
   XtOffsetOf(ResData,snoselected),XtRString,
   String("You must either select human or computer!")},
  {"openfile","Openfile",XtRString,sizeof(String),
   XtOffsetOf(ResData,sopenfile),XtRString,
   String("Open file")},
  {"savefile","Savefile",XtRString,sizeof(String),
   XtOffsetOf(ResData,ssavefile),XtRString,
   String("Save file")},
  {"colorwhite","Colorwhite",XtRString,sizeof(String),
   XtOffsetOf(ResData,scolorwhite),XtRString,String("white")},
  {"colorblack","Colorblack",XtRString,sizeof(String),
   XtOffsetOf(ResData,scolorblack),XtRString,String("black")},
  {"backgroundboard","Backgroundboard",XtRString,sizeof(String),
   XtOffsetOf(ResData,sbackgroundboard),XtRString,String("grey")},
  {"blackfieldcolor","Blackfieldcolor",XtRString,sizeof(String),
   XtOffsetOf(ResData,sblackfieldcolor),XtRString,String("blue")},
  {"whitefieldcolor","Whitefieldcolor",XtRString,sizeof(String),
   XtOffsetOf(ResData,swhitefieldcolor),XtRString,String("green")}
};


//*************************** loadfilesel *************************
/**
 * creates a file requester
 * 
 * @param title  string to disply in the title of the file requestor
 */
void loadfilesel(char* title=NULL)
{
  // create file selector
  //fileselpopup=XtVaCreatePopupShell("fileselectorpopup",transientShellWidgetClass,toplevel,NULL);
  filesel=XtVaCreateManagedWidget("fileselector",fileNominatorWidgetClass,fileselpopup,NULL);
  if (title!=NULL) XtVaSetValues(fileselpopup,XtNtitle,title,NULL);

  // ok and cancel callbacks
  XtAddCallback(filesel,XtNselectCallback,loadorsave,NULL);
  XtAddCallback(filesel,XtNcancelCallback,oncancelfiledialog,NULL);

  //realize widget
  XtRealizeWidget(fileselpopup);
}


//******************************* thread2 **************************************
void* thread2(void* p)
{
  FILE* tinput;
  FILE* toutput;
  SharedData* tsd=(SharedData*)p; 
  tinput=fdopen((tsd->pipe_tothread)[0],"r");
  toutput=fdopen((tsd->pipe_toapp)[1],"w");
  char indata;
  
  for (;;)
  {
    indata=fgetc(tinput);
    if (indata==startcalcmove)
    {
      tsd->bestmove=board.besterzug(cs.m_sp,tsd->command,cs.m_level[cs.m_sp]);
      fputc(ready,toutput);
      fflush(toutput);
    }
    else if (indata==quittask)
    {
      fclose(tinput);
      fclose(toutput);
      break;
    }
  }
  return 0;
}


//************************* handledrag *********************************
/* 
 * should be called, if you start a drag operaton
 * 
 * @param winx  x coordinate of pointer position
 * @param winy  y coordinate of pointer position
 */
void handledrag(int winx,int winy)
{
        
	//Computer zieht gerade?
	if (cs.m_bcomputermove) return;


	//innerhalb erlaubtem Bereich?
        xx1=boardview.convwinxtox(winx,false,0);
        yy1=boardview.convwinytoy(winy,false,0);
	if (xx1<0 || xx1>xmax+2 || yy1<1 || yy1>ymax+1) return;
	if (xx1==0 || xx1==xmax+2) bmove=false;
	   else bmove=true;

	//steht dort eine Figur ?
	if (boardview.getcontents(xx1,yy1)==NULL) return;

	//Setupposition?
	if (!(bsetuppos)) 
	{
		//hat die Figur die richtige Farbe?
		if (cs.m_sp!=spvonstein(board.getcontents(xx1-1,yy1-1))) return;
			
		// Spielfortsetzung mglich?
		if (cs.m_gamestate==matt || cs.m_gamestate==patt || 
                    cs.m_gamestate==istremis) return;
               
	}
        
	

	//Drag-Prozedur einleiten
        boardview.begindrag(winx,winy,xx1,yy1,bmove);
        bdrag=true;	     
}

//********************** handledrop *******************
/* 
 * should be called if you end a drag operation
 * 
 * @param winx  x coordinate of poiter position
 * @param winy  y coordinate of pointe position
 *
 * @return true, if operation is valid, else otherwise
 */ 
bool handledrop(int winx,int winy)
{
	// Drag-Operation beenden
	boardview.drop(winx,winy,xx2,yy2);
        bdrag=false;
	
	CCounter* ss=boardview.getcontents(xx1,yy1);

	// Stein auf sich selber fallen lassen?
	if (xx1==xx2 && yy1==yy2)
	{
	   ss->draw();
	   return false;
	}
	
	//Copy?
	if (!bmove)
	{
	        // copy operation outside the board nothing is to do
                if (!(cs.isinside(xx2,yy2))) return true;

		//normal copy operation
               	ss->clone(xx2,yy2);
		return true;
	}
	else if (bsetuppos) //Move && Setupposition
	{
			
			// move operation outside the window deletes the source piece
			if (!(cs.isinside(xx2,yy2)))  delete ss;

			// otherwise moves the source piece
			else ss->smove(xx2,yy2,false);
			return true;
	}
	else 
	{  //normaler Move

		//auerhalb Spielfeld
		if (!cs.isinside(xx2,yy2))
		{
		   ss->draw();
		   return false;
		}

		//Zug mglich?
		zug z={xx1-1,yy1-1,xx2-1,yy2-1};
		if (!(board.moeglich(z,cs.m_sp)))
		{
		   ss->draw();
		   return false;
		}
               
                // transformation of pawn
		if ((z.y2==0 || z.y2==ymax ) && abs(ss->getvalue())==bauer)
		{
		  cs.popupiconsel(toplevel,z);
		  return true;
		}
		cs.smove(z,false); //Sourcefeld nicht lschen

		// anschlieend Computerzug?
		if (cs.m_level[cs.m_sp]>0 && 
			(cs.m_gamestate==normal || cs.m_gamestate==istschach))
                            cs.start_computermove();
	}
	return true;
}

// ************************* getthreadinput ****************************
void getthreadinput(XtPointer clientdata,int *fid,XtInputId *id)
{
  char indata[1];
  
  int nbytes=read(*fid,indata,1);
  if (nbytes==-1)
    cout << "Error" << endl;
  cs.do_computermove();
}

//*************************** loadgame ****************************
/**
 * load game with name str
 *
 * @param str  filename to load
 */
void loadgame(char* str)
{
  ifstream f(str);
  if (f==0)
  {
     popupMessageBox(res.serror,"Could not open file!");
     return;
  }
  try
  {
    f >> cs;
  }

  // invalid filetyp
  catch (const char* p)
  {
    popupMessageBox(res.serror,p);
    return;
  }

  // close fileselector
  if (filesel) 
  {
    XtPopdown(fileselpopup);
    XtDestroyWidget(filesel);
  }
  cs.update();
  strcpy(game_name,str);
}

//***************************** savegame ****************************
/** 
 * save game 
 *
 * @param str  filename to save
 * @param force  if true, overwrite file if it exists already, 
 *               if false, open a requester, if file should be overwritten
 */
void savegame(char* str,bool force=false)
{
  if (!force)
  {
    ifstream inf(str);
    if (inf!=0)
    {
      inf.close();
      if (ocpopup==0) //not pop'd up before
      {
          createokcancelmsgbox(toplevel,ocpopup,ocform,oclabel,ocok,occancel);
          XtAddCallback(ocok,XtNcallback,onoverwriteok,NULL);
          XtAddCallback(occancel,XtNcallback,onoverwritecancel,NULL);
      }
      strncpy(game_nameoverwrite,str,511);
      XtPopup(ocpopup,XtGrabNonexclusive);
      return;
    }
  }
  ofstream f(str);
  if (f==0)
  {
    popupMessageBox(res.serror,"Could not open file!");
    return;
  }
  XtPopdown(fileselpopup);
  XtDestroyWidget(filesel);
  f  << cs;

  //set file name
  strcpy(game_name,str);

  //computermove
  if (!bsetuppos && cs.m_level[cs.m_sp]>0) cs.start_computermove();
}

//********************** setmenus ******************************
void setmenus(Boolean setuppos)
{
  XtVaSetValues(game_new,XtNsensitive,!setuppos,NULL);
  XtVaSetValues(game_load,XtNsensitive,!setuppos,NULL);
  XtVaSetValues(takeback,XtNsensitive,!setuppos,NULL);
  XtVaSetValues(lastown,XtNsensitive,!setuppos,NULL);
  XtVaSetValues(clearboard,XtNsensitive,setuppos,NULL);
}

//***************************** callbacks ******************************

//************************ onoverwriteok *******************************
void onoverwriteok(Widget w,XtPointer client,XtPointer call)
{
  XtPopdown(ocpopup);
  savegame(game_nameoverwrite,true);
}

//****************************** onoverwritecancel ***************************
void onoverwritecancel(Widget w,XtPointer client,XtPointer call)
{
  XtPopdown(ocpopup);
}


//****************************** loadorsave *******************************
void loadorsave(Widget w,XtPointer client,XtPointer call)
{
  //string name;
  char name[512];
  FileNominatorStruct* pfns=(FileNominatorStruct*)call;

  if (strlen(pfns->directoryPart)+strlen(pfns->filenamePart)>511)
  {
    popupMessageBox(res.serror,"filename too long!");
    return;
  }

  strcpy(name,pfns->directoryPart);
  strcat(name,pfns->filenamePart);
  if (bload) 
    loadgame(name);

  else savegame(name);
}

//******************************* oncancelfiledialog ************************
void oncancelfiledialog(Widget w,XtPointer client,XtPointer call)
{
  XtPopdown(fileselpopup);
  if (cs.m_level[cs.m_sp]>0) cs.start_computermove();
}

//****************************** newgame ************************************
void newgame(Widget w,XtPointer client,XtPointer call)
{
  if (bsetuppos) return;
  cs.newgame();
}

//********************************* onsavegame *********************************
/* will be called if the user chooses save from the game menu */
void onsavegame(Widget w,XtPointer client,XtPointer call)
{
  if (cs.m_bcomputermove) cs.abort_computermove();
  bload=false;
  if (strcmp(game_name,"")!=0) 
  {
    savegame(game_name,true);
    return;
  } 
  //load fileselector if necessary
  
  //if (fileselpopup==0) loadfilesel();
  loadfilesel(res.ssavefile); //neu
  //pop up fileselector
  XtRealizeWidget(filesel);
  XtPopup(fileselpopup,XtGrabNonexclusive);
}

//********************************* onsavegameas ******************************
void onsavegameas(Widget W,XtPointer client,XtPointer call)
{
  // abort computermove
  if (cs.m_bcomputermove) cs.abort_computermove();

  //load fileselector if necessary
  //if (fileselpopup==0) loadfilesel();
  loadfilesel(res.ssavefile);

  //popup fileselector
  bload=false;
  XtRealizeWidget(filesel);
  XtPopup(fileselpopup,XtGrabNonexclusive);
}

//********************************* onloadgame ********************************
void onloadgame(Widget w,XtPointer client,XtPointer call)
{
  if (bsetuppos) return;

  //load fileselector if necessary
  //if (fileselpopup==0) loadfilesel();
  loadfilesel(res.sopenfile);

  //popup fileselector
  bload=true;
  XtRealizeWidget(filesel);
  XtPopup(fileselpopup,XtGrabNonexclusive);
}

// ******************************* quit **************************************
void quit(Widget w,XtPointer client,XtPointer call)
{
  XtDestroyApplicationContext(app_context);
  cs.sendmessage(quittask);
  exit(0);
}


//***************************** setupplayers ********************************
void setupplayers(Widget w,XtPointer client,XtPointer call)
{
  if (bsetuppos) return;
  if (cs.m_bcomputermove) cs.abort_computermove();
  popupplayersetupdialog(cs.m_level[0],cs.m_level[1]);
}

//***************************** ontakeback ***********************************
void ontakeback(Widget w,XtPointer client,XtPointer call)
{
  if (bsetuppos) return;
  if (cs.m_bcomputermove) cs.abort_computermove();
  cs.takebackmove();
  if (cs.m_level[cs.m_sp]>0) cs.start_computermove();
}

//**************************** onlastown ********************************
void onlastown(Widget w,XtPointer client,XtPointer call)
{
  if (bsetuppos) return;
  // possible
  if (!(cs.m_level[cs.m_sp]==0 ^ cs.m_level[!cs.m_sp]==0)) return; // exactly one human 

  if (cs.m_bcomputermove) cs.abort_computermove();
	do
	{
	  cs.takebackmove(false); //not updating
	}	while (cs.m_level[cs.m_sp]!=0);
	if (cs.m_level[cs.m_sp]>0) cs.start_computermove();
  //update
  cs.m_gamestate=board.analyse(cs.m_sp);
  cs.m_lastmove=board.holezug(board.hz-1);
  cs.updatestatuswindow();
}

//**************************** onsetuppos ***********************************
void onsetuppos(Widget W,XtPointer client,XtPointer call)
{
  int x,y,s;
  CCounter* ss;
  if (bsetuppos)
  {

    //adjust board to visual board
    for (x=0;x<=xmax;x++)
    {
      for (y=0;y<=ymax;y++)
      {
	ss=boardview.getcontents(x+1,y+1);
	if (ss) f[x][y]=ss->getvalue();
	  else f[x][y]=0;
      }
    }
    if (!(board.setzebrett(f))) 
    {
      popupMessageBox(res.serror,res.sinvbo);
      return;
    }
    
    if ((board.schach(!cs.m_sp))) 
    {
      popupMessageBox(res.serror,res.sinvbo);
      return;
    }
    setmenus(False);
    bsetuppos=false;
    uncheckmenuitem(setuppos);
    XtRealizeWidget(menu_board);

    // delete pieces outside the board
    for (y=1;y<=7;y++)
    {
      ss=boardview.getcontents(0,y);
      delete ss;
      ss=boardview.getcontents(xmax+2,y);
      delete ss;
    }  
  }
  else //setup position
  {
    bsetuppos=true;    
    checkmenuitem(setuppos);
    XtRealizeWidget(menu_board);
    setmenus(True);
    //create pieces outside the board
    for (s=turm,y=1;s<=bauer;s++,y++)
    {
      boardview.setcontents(0,y,new CCounter(0,y,cs.convvaluetoicon(s),s,&boardview));
      boardview.setcontents(xmax+2,y,new CCounter(xmax+2,y,cs.convvaluetoicon(-s),-s,&boardview));
    }
  }
}	

//**************************** onclearboard *******************************
void onclearboard(Widget w,XtPointer client,XtPointer call)
{
  if (!(bsetuppos)) return;
  boardview.eraseboard();
  boardview.draw();
}

//*************************** onhelp **************************************
void onhelp(Widget w,XtPointer client,XtPointer call)
{
#ifdef USE_STRSTREAM
  char s[256];
  ostrstream os(s,255);
#else
  ostringstream os;
#endif
  if (strlen(res.shelpapp)+strlen(res.shelpfile)>250) return;
  os << res.shelpapp << ' ' << prgDir << res.shelpfile << " &" << '\0';
#ifdef USE_STRSTREAM
  cout << "Execute " << s << endl;
  system(s);
#else
  string s(os.str());
  cout << "Execute " << s << endl;
  system(s.c_str());
#endif
}
  

//*********************** end callbacks **************************************


//*********************************** events **************************************

//******************************* whiteisplay **************************************
void whiteisplay(Widget w,XtPointer client,XEvent* ev,Boolean* b)
{
  //boardview.draw(XtWindow(w));
  if (bfirsttime) boardview.setwindow(XtWindow(w));
  bfirsttime=false;
  boardview.draw();
}  
 
//************************ press_event ******************************
void press_event(Widget w,XtPointer client,XEvent *ev,Boolean* b)
{
  if ((ev->xbutton).button==Button1)
    handledrag((ev->xbutton).x,(ev->xbutton).y);
  else if ((ev->xbutton).button==Button2)
  {
     if (!(bdrag)) handledrag((ev->xbutton).x,(ev->xbutton).y);
     else
     {
        handledrop((ev->xbutton).x,(ev->xbutton).y);
        bdrag=false;
	bmove=true;
     }
  }
}

//******************** motion_event **********************
void motion_event(Widget w,XtPointer client,XEvent *ev,Boolean* b)
{
   if (bdrag==true) boardview.dodrag((ev->xbutton).x,(ev->xbutton).y);
} 

//********************** release_event *******************************
void release_event(Widget w,XtPointer client,XEvent *ev,Boolean* b)
{
  if (bdrag==true && (ev->xbutton).button==Button1) 
  {
    handledrop((ev->xbutton).x,(ev->xbutton).y);
    bdrag=false;
    bmove=true;
  }
}

//*********************** configure_event ******************************
void configure_event(Widget w,XtPointer client,XEvent *ev,Boolean* b)
{
   Dimension width,height;
   if (!bfirstconf)
   {
     XtVaGetValues(w,XtNwidth,&width,XtNheight,&height);
     boardview.resize(width,height);
   }
   bfirstconf=false;
} 

//************************** end events ******************************

//*************************** actions ********************************
static void anewgame(Widget w,XEvent *event,String* params,Cardinal* numparams)
{
  newgame(w,NULL,NULL);
}

static void asavegame(Widget w,XEvent *event,String* params,Cardinal* numparams)
{
  onsavegame(w,NULL,NULL);
}

static void asavegameas(Widget w,XEvent *event,String* params,Cardinal* numparams)
{
  onsavegameas(w,NULL,NULL);
}

static void aloadgame(Widget w,XEvent *event,String* params,Cardinal* numparams)
{
  onloadgame(w,NULL,NULL);
}

static void aquit(Widget w,XEvent *event,String* params,Cardinal* numparams)
{
  quit(w,NULL,NULL);
}

static void asetupplayers
   (Widget w,XEvent *event,String* params,Cardinal* numparams)
{
  setupplayers(w,NULL,NULL);
}

static void atakeback(Widget w,XEvent *event,String* params,Cardinal* numparams)
{
  ontakeback(w,NULL,NULL);
}

static void alastown(Widget w,XEvent *event,String* params,Cardinal* numparams)
{
  onlastown(w,NULL,NULL);
}

static void asetuppos(Widget w,XEvent *event,String* params,Cardinal* numparams)
{
  onsetuppos(w,NULL,NULL);
}

static void aclearboard(Widget w,XEvent *event,String* params,Cardinal* numparams)
{
  onclearboard(w,NULL,NULL);
}


//****************************** end actions *******************************



static XtActionsRec actions[]=
{
  {"newgame",anewgame},
  {"savegame",asavegame},
  {"savegameas",asavegameas},
  {"loadgame",aloadgame},
  {"quit",aquit},
  {"setupplayers",asetupplayers},
  {"takeback",atakeback},
  {"lastown",alastown},
  {"setuppos",asetuppos},
  {"clearboard",aclearboard}
};

//******************************* main **************************************
int main(int argc,char** argv)
{
  Dimension boxwidth=0,boxheight=0;
  pthread_t thread_id;

  XtSetLanguageProc(NULL,(XtLanguageProc)NULL,NULL);
  toplevel=XtVaAppInitialize(&app_context,"Jchess",NULL,0,&argc,argv,NULL,
    XtNinput,True,NULL);
  XtVaGetApplicationResources(toplevel,&res,resources,XtNumber(resources),NULL);
  XtAppAddActions(app_context,actions,XtNumber(actions));
  
  //get Programm dir
  prgDir=getPrgDir(argv[0])+"/";

  // create composite window with one box and a menu
  form=XtVaCreateManagedWidget("form",formWidgetClass,toplevel,NULL);

 

  // create menu
  button_partie=XtVaCreateManagedWidget("game",menuButtonWidgetClass,form,
          XtNmenuName,"menu1",NULL);
  menu_partie=XtCreatePopupShell("menu1",simpleMenuWidgetClass,button_partie,NULL,0);
  game_new=XtVaCreateManagedWidget("new",smeBSBObjectClass,menu_partie,NULL);
  game_load=XtVaCreateManagedWidget("open",smeBSBObjectClass,menu_partie,NULL);
  game_save=XtVaCreateManagedWidget("save",smeBSBObjectClass,menu_partie,NULL);
  game_saveas=XtVaCreateManagedWidget("saveas",smeBSBObjectClass,menu_partie,NULL);
  game_quit=XtVaCreateManagedWidget("quit",smeBSBObjectClass,menu_partie,NULL);
  players=XtVaCreateManagedWidget("setupplayers",commandWidgetClass,form,XtNfromHoriz,button_partie,NULL);
  button_move=XtVaCreateManagedWidget("button_move",menuButtonWidgetClass,form,XtNmenuName,"menu_move",
              XtNfromHoriz,players,NULL);
  menu_move=XtVaCreatePopupShell("menu_move",simpleMenuWidgetClass,button_move,NULL);
  takeback=XtVaCreateManagedWidget("takeback",smeBSBObjectClass,menu_move,NULL);
  lastown=XtVaCreateManagedWidget("lastown",smeBSBObjectClass,menu_move,NULL);
  button_board=XtVaCreateManagedWidget("button_board",menuButtonWidgetClass,form,
         XtNfromHoriz,button_move,XtNmenuName,"menu_board",NULL);
  menu_board=XtVaCreatePopupShell("menu_board",simpleMenuWidgetClass,button_board,NULL);
  setuppos=XtVaCreateManagedWidget("setuppos",smeBSBObjectClass,menu_board,
                         NULL);
  clearboard=XtVaCreateManagedWidget("clearboard",smeBSBObjectClass,menu_board,NULL);
  help=XtVaCreateManagedWidget("help",commandWidgetClass,form,XtNfromHoriz,button_board,NULL);
  XtAddCallback(game_new,XtNcallback,newgame,NULL);
  XtAddCallback(game_load,XtNcallback,onloadgame,NULL);
  XtAddCallback(game_save,XtNcallback,onsavegame,NULL);
  XtAddCallback(game_saveas,XtNcallback,onsavegameas,NULL);
  XtAddCallback(game_quit,XtNcallback,quit,NULL);
  XtAddCallback(players,XtNcallback,setupplayers,NULL);
  XtAddCallback(takeback,XtNcallback,ontakeback,NULL);
  XtAddCallback(lastown,XtNcallback,onlastown,NULL);
  XtAddCallback(setuppos,XtNcallback,onsetuppos,NULL);
  XtAddCallback(clearboard,XtNcallback,onclearboard,NULL);
  XtAddCallback(help,XtNcallback,onhelp,NULL);

  //init dialog
  initplayersetupdialog(toplevel,app_context,&cs);
  initMessageBox(toplevel,app_context);
  fileselpopup=XtVaCreatePopupShell("fileselectorpopup",
      transientShellWidgetClass,toplevel,XtNinput,True,NULL); 

  // create box
  //box=XtVaCreateManagedWidget("box",boxWidgetClass,toplevel,XtNwidth,200,XtNheight,200,NULL);
  wbox=XtVaCreateManagedWidget("box",boxWidgetClass,form,XtNfromVert,button_partie,NULL);

  //create statuswindow
  statuswindow=XtCreatePopupShell("statuswindow",topLevelShellWidgetClass,toplevel,NULL,0);
  label=XtVaCreateManagedWidget("label",labelWidgetClass,statuswindow,NULL);
  XtPopup(statuswindow,XtGrabNone);

  // initialize thread2
  SharedData* sd=new SharedData;
  sd->command=0;
  if (pipe(sd->pipe_tothread)) displayfileerror("Pipe","main",true);
  if (pipe(sd->pipe_toapp)) displayfileerror("Pipe","main",true);
  //ainput=fdopen((sd->pipe_toapp)[0],"r");
  pthread_create(&thread_id,NULL,thread2,sd);
 
  // add event handlers
  XtAddEventHandler(wbox,ExposureMask,FALSE,whiteisplay,NULL); 
  XtAddEventHandler(wbox,ButtonPressMask,FALSE,press_event,NULL);
  XtAddEventHandler(wbox,PointerMotionMask,FALSE,motion_event,NULL);
  XtAddEventHandler(wbox,ButtonReleaseMask,FALSE,release_event,NULL);
  XtAddEventHandler(wbox,StructureNotifyMask,FALSE,configure_event,NULL);

  //init Cjchess-object
  XtVaGetValues(wbox,XtNheight,&boxheight,XtNwidth,&boxwidth,NULL);
  display=XtDisplay(wbox);
  cs.create(&board,&boardview,display,boxwidth,boxheight,sd,&res,label,takeback,lastown);

  XtAppAddInput(app_context,fileno(cs.m_input),(void*)XtInputReadMask,getthreadinput,NULL);

  //load icon 
  XpmAttributes attr;
  Pixmap iconpixmap;
  attr.valuemask=0;
  string iconFilename=prgDir+"janus.Xpm";
  int status=XpmReadFileToPixmap(display,
				 RootWindow(display,DefaultScreen(display)),
				 (char*)iconFilename.c_str(),
				 &iconpixmap,
				 NULL,
				 &attr);
  if (status!=XpmSuccess) displayfileerror((char*)iconFilename.c_str(),"main",true);
  XtVaSetValues(toplevel,XtNiconPixmap,iconpixmap,NULL);

  //load file
  if (argc>1) loadgame(argv[1]); 
  
  XtRealizeWidget(toplevel);
  XtAppMainLoop(app_context);
  return 0;
}





