/* brett.cpp*/
#include <math.h>
#include <iostream>
#include <stdlib.h>
#include <time.h>
#include <ctype.h>
#include "jsbrett.h"
#include <vbase/array.h>



/*************** operator<< fr Zug ***********/
ostream& operator<<(ostream& os,const zug& z)
{
  char x;

  switch (abs(z.bef))
  {
  case dame: x='Q';break;
  case janus: x='J';break;
  case pferd: x='H';break;
  case turm: x='R';break;
  case laeufer: x='B';break;
  default: x='-';
  }
  if (z.bef<0) x=(char)tolower(x);
  os << char(z.x1+'A') << z.y1+1 << x
     << char(z.x2+'A') << z.y2+1;
  return os;
}

/**************** operator>> fr Zug************/
istream& operator>>(istream& is,zug& z)
{
  char c,c1,c2;

  is >> c1 >> z.y1 >> c >> c2 >> z.y2;
  z.x1=int(c1-'A');
  z.x2=int(c2-'A');
  z.y1--;
  z.y2--;
  switch (toupper(c))
  {
  case 'Q': z.bef=dame;break;
  case 'R': z.bef=turm;break;
  case 'H': z.bef=pferd;break;
  case 'J': z.bef=janus;break;
  case 'B': z.bef=laeufer;break;
  default: z.bef=0;
  }
  if (islower(c)) z.bef=-z.bef;
  return is;
}

/******************* BERAPHABETA ******************/
void beralphabeta(const int&i,int& alpha,int& beta)
{
  alpha=((i-1)/4+1)*2-1;
  beta=alpha+((i-1)%4)*2;
}

/*************** Konstruktor *******************/
jsbrettklasse::jsbrettklasse()
{
  zi=NULL; //Speicher fr Zge
  ind=NULL; //Zugriff ber Indexfeld (zum Sortieren)
  bew=NULL; //Speicher fr Bewertungen
}

/*************** Desktruktor *******************/
jsbrettklasse::~jsbrettklasse()
{
  FreeMatrix(ind,anztiefen);
  FreeMatrix(bew,anztiefen);
  FreeMatrix(zi,anztiefen);
}

//*********************** INIT ********************************
void jsbrettklasse::init(int js){

int x,y;
//cout << endl;
jan=js;
hz=0;
koenigx[0]=4; //Knigsposition speichern
koenigy[0]=0;
koenigx[1]=4;
koenigy[1]=ymax;
rochrx=8; //Zu welchem Feld Rochade?
rochlx=1;
rturmbew[0]=rturmbew[1]=lturmbew[0]=lturmbew[1]=-1;
koenigbew[0]=koenigbew[1]=-1; //Knig noch nicht bewegt
pu[0]=pu[1]=12500; //Punkte aller Steine
gl[0]=0; //Grundlinien
gl[1]=ymax;

//Spielfeld lschen
for (x=0;x<=xmax ;x++ ) {
  for (y=1;y<ymax ;y++ ) f[x][y]=leer;
}

//Grundlinie initialisieren

//Turm
f[0][0]=f[xmax][0]=turm;
f[0][ymax]=f[xmax][ymax]=-turm;

//Janusfigur
if (jan)
{
  f[1][0]=f[8][0]=janus;
  f[1][ymax]=f[8][ymax]=-janus;
}

//Pferd
f[1+jan][0]=f[6+jan][0]=pferd;
f[1+jan][ymax]=f[6+jan][ymax]=-pferd;

//Laeufer
f[2+jan][0]=f[5+jan][0]=laeufer;
f[2+jan][ymax]=f[5+jan][ymax]=-laeufer;

//Dame
f[3+jan+jan][0]=dame;
f[3+jan+jan][ymax]=-dame;

//Koenig
f[4][0]=koenig;
f[4][ymax]=-koenig;

//Bauern
for (x=0;x<=xmax;x++)
{
  f[x][1]=bauer;
  f[x][ymax-1]=-bauer;
}

}

/***************** MOEGLICHFUERKOENIG ********************/
bool jsbrettklasse::moeglichfuerkoenig(const zug& z)
{
  int sp=spvonstein(f[z.x1][z.y1]);

  if (abs(z.x2-z.x1)<=1 && abs(z.y2-z.y1)<=1) return true;
  if (z.y1!=z.y2) return false;
  if (z.x1==4 && z.x2==rochrx && moeglichrochade(sp,1)) return true;
  if (z.x1==4 && z.x2==rochlx && moeglichrochade(sp,-1)) return true;
  return false;
}

/***************** MOEGLICHFUERTURM *********************/
bool jsbrettklasse::moeglichfuerturm(const zug& z)
{
  int x,y,sx=sgn(z.x2-z.x1),sy=sgn(z.y2-z.y1);

  if (sx!=0 && sy!=0) return false;
  for (x=z.x1+sx,y=z.y1+sy;x!=z.x2 || y!=z.y2;x=x+sx,y=y+sy)
    if (f[x][y]!=leer) return false;
  return true;
}

/***************** MOEGLICHFUERLAEUFER *******************/
bool jsbrettklasse::moeglichfuerlaeufer(const zug& z)
{
  int x,y,sx=sgn(z.x2-z.x1),sy=sgn(z.y2-z.y1);

  if (abs(z.x2-z.x1)!=abs(z.y2-z.y1)) return false;
  for (x=z.x1+sx,y=z.y1+sy;x!=z.x2;x=x+sx,y=y+sy)
    if (f[x][y]!=leer) return false;
  return true;
}

/******************* MOEGLICHFUERPFERD*******************/
bool jsbrettklasse::moeglichfuerpferd(const zug& z)
{
  int dx=z.x2-z.x1,dy=z.y2-z.y1;

  return (abs(dx)==2 && abs(dy)==1 || abs(dx)==1 && abs(dy)==2);
}

/****************** MOEGLICHFUERBAUER ********************/
bool jsbrettklasse::moeglichfuerbauer(const zug& z)
{
  int k=0,t=0,i;
  zug z2=z,zv;

  /* alle Zuege von z.x1,z.y1, die mit Bauer mglich sind
     ermitteln, und anschlieend prfen, ob dieser Zug dabei*/
  if (hz==0) zv=leererzug;
  else zv=zr[hz-1];
  zuegevonbauer(t,k,z2,true,false,NULL,NULL,zv);
  for (i=0;i<k;i++)
  {
    if (z==zi[0][i]) return true;
  }
  return false;
}


/***************** MOEGLICHVONFIGUR **********************/
/* berprft, ob ein Zug mglich ist von der Art
wie sich der betreffende Stein generell bewegen kann*/

bool jsbrettklasse::moeglichvonfigur(const zug& z){
int dx=z.x2-z.x1,dy=z.y2-z.y1,sx=sgn(dx),sy=sgn(dy);
bool sprung=false;
switch (abs(f[z.x1][z.y1])) {
 case koenig:
   return moeglichfuerkoenig(z);
 case laeufer:
   return moeglichfuerlaeufer(z);
 case turm:
   return moeglichfuerturm(z);
 case pferd:
   return moeglichfuerpferd(z);
 case bauer:
   return moeglichfuerbauer(z);
 case janus:
   return (moeglichfuerpferd(z) || moeglichfuerlaeufer(z));
 case dame:
   return (moeglichfuerlaeufer(z) || moeglichfuerturm(z));
} //endeswitch
abort();
return 0;
} //endemoeglichvonfigur

/************************ SCHACH **************************/
bool jsbrettklasse::schach(const int& sp)
{
/* berprft ob Spieler sp im Schach ist*/
  zug z;
  stein h;

  z.x2=koenigx[sp];
  z.y2=koenigy[sp];
  z.bef=0;
  z.ep=false;

  for (z.x1=0;z.x1<=xmax;z.x1++)
  {
    for (z.y1=0;z.y1<=ymax;z.y1++)
    {
      h=f[z.x1][z.y1];
      if (h!=leer && spvonstein(h)!=sp)
      {
        if (abs(h)==bauer)
        {
          if (abs(z.x2-z.x1)==1 && z.y2-z.y1==sgn(h))
            return true;
        }
        else if (moeglichvonfigur(z)) return true;
      }
    }
  }
  return false;
}

//************************* REMIS ***************************
bool jsbrettklasse::remis() {
for (int x=0;x<=xmax ;x++ ){
  for (int y=0;y<=ymax ;y++ ){
    stein h=abs(f[x][y]);
    if (h!=leer && h!=koenig)
       return false;
  }
}
return true;
}

//******************** ALLGMOEGLICH *************************
bool jsbrettklasse::allgmoeglich(const zug& z)
{
/* berprft ab ein bei einem Zug kein eigener Stein auf dem
Zielfeld steht */
  //auf Zielfeld steht eigener Stein:
  return !(sgn(f[z.x1][z.y1])==sgn(f[z.x2][z.y2]));
}


//************************ ZUEGEVONSPIELER ***********************
int jsbrettklasse::zuegevonspieler
      (const int& sp,long* b0,bool schrbew,bool vollesuche,
       const int&t,int *ix){
int x,y,k=0;
for (x=0;x<=xmax ;x++ ) {
  for (y=0;y<=ymax ;y++ ) {
    if (spvonstein(f[x][y])==sp && f[x][y]!=leer) {
         //Feld ist von Spieler sp besetzt?
      if(!zuegevonfeld(x,y,t,k,b0,schrbew,vollesuche,ix))
      {
        zi[t][0]=leererzug; //Mensch-Optimierung durchgefhrt
        return -1;
      }
    } /* endif */
  } /* endfor */
} /* endfor */
zi[t][k]=leererzug;
return k;
}

//***************************** ANALYSE ********************
zctyp jsbrettklasse::analyse(const int& sp,int t) {
int anz,i;
bool pattmatt=true;
stein h;
if (remis()) return istremis;
anz=zuegevonspieler(!sp,NULL,false,true,t,NULL);

for (i=0;i<anz;i++) {
  h=ziehe(zi[t][i]);
  if (!schach(!sp)) {
    pattmatt=false;
    zuruecknehmen(zi[t][i],h);
    break;
  }
  zuruecknehmen(zi[t][i],h);
}
if (pattmatt) {
  if (schach(!sp)) return matt;
  return patt;
}
else {
  if (schach(!sp)) return istschach;
}
return normal;
}


/******************* besterzug ************************/
/* i[t] bedeutet Beim wievielt besten Zug befindet man sich in
        Tiefe t
   ind[t][h]: in Tiefe t ist der folgende der h.-beste Zug*/
zug jsbrettklasse::besterzug
    (int sp,int& command,int lev)
{
  long b0[60];
  register int l;
  int t=0,l2,index,k,k0,i[60]; //i:Wo bin ich bei Tiefe t
  zug az,bestzug;
  stein hi[60],zfi; //Speichere geschlagene Figuren,Zielfeldinhalt
  int startsp=sp; //Spieler an der Reihe
  bool opt;

  // Initialisierung
  if (lev==0) lev=level;
  beralphabeta(lev,alpha0,beta0);
  alpha=alpha0; //Alpha-Suchtiefe auf Standardwert
  beta=beta0;  //Beta-Suchtiefe auf Standardwert
  b0[0]=-unendl;
  i[0]=0;
  k0=zuegevonspieler(sp,NULL,true,true,0,i);

  //Verhindere immer dieselben Zge
  if (hz>=4)
    for (int loop=0;loop<k0;loop++)
      if (zi[0][loop]==zr[hz-4]) bew[0][loop]-=20;

  bew[t][time(NULL)%k0]+=5+70*(hz<4);

  //Rochade
  if (moeglichrochade(sp,-1))
  {
    zi[0][k0].x1=4;
    zi[0][k0].y1=gl[sp];
    zi[0][k0].x2=rochlx;
    zi[0][k0].y2=gl[sp];
    bew[0][k0++]=30;
  }
  if (moeglichrochade(sp,1))
  {
    zi[0][k0].x1=4;
    zi[0][k0].y1=gl[sp];
    zi[0][k0].x2=rochrx;
    zi[0][k0].y2=gl[sp];
    bew[0][k0++]=30;
  }
  zi[0][k0]=leererzug;

  ordnen(k0,ind[0],bew[0]);
  //Zge von Spieler sp, keine Bewertungsoptimierung
  //Bewertungen speichern,alle Zge durchrechnen

  for (l=1;l<anztiefen;l=l+2) //Fr alle ungeraden Indizes
    for (l2=0;l2<=mzugm;l2++) ind[l][l2]=l2;

 
  for (;;)
  {
    // Bestimme zfi,index und az
    index=ind[t][i[t]];
    az=zi[t][index]; //aktueller Zug

    
    if (az!=leererzug)  zfi=abs(f[az.x2][az.y2]); //Zielfeldinhalt
    else zfi=0;

    if (az!=leererzug && t<beta && zfi!=koenig)
    /******************** Eins hher ***********************/
    {

      //Zug abbrechen
      if (command>=100)
      {
	for (l=t-1;l>=0;l--)
          zuruecknehmen(zi[l][ind[l][i[l]]],hi[l]);
        return leererzug;
      }

      /* Computeroptimierung
         Man ermittle den letzten Zug des Computers.
         Ist dort eine Bewertung niedriger als eine fertige
         Bewertung wei man, da dieser Zug nur noch schlechter
         werden kann, weil im nchsten Zug der Mensch zum
         letzten Mal zieht*/
      opt=false;
      if (t==beta-1 || t==alpha-1 && b0[t]>0 && pu[!sp]>7300)
      {
        for (l=0;l<=t;l=l+2)
        {
          if (bew[t][index]<=b0[l])
          {
            opt=true;
            break;
          }
        }
      }
      /* Ende Computer-Optimierung */

      if (opt) zi[t][index]=leererzug;
      else
      {

        //eins rauf
        hi[t]=ziehe(zi[t][index]);
        t++; //Tiefe erhhen
        i[t]=0; //Wieder von vorne anfangen
        sp=!sp;

        //Ermittle alle mglichsten nchsten Zge
        k=zuegevonspieler(sp,b0,true,t<=alpha,t,i);
        // Zge von Spieler sp,
        // Bewertungen speichern, evtl. alle Zge ermitteln

	// Zugmglichkeitsstrategie
        if (k>0 && k<40) bew[t-1][ind[t-1][i[t-1]]]+=40-k;

        /* 1.Fall k=-1: Mensch-Optimierung,d.h. Vorgngerzug
           wird auf keinen Fall gewhlt*/
        if (k==-1) b0[t]=unendl; //Mensch-Optimierung

        /* 2.Fall k=0 aber nicht alle Zge wurden berprft*/
        else if (t>alpha && !k) b0[t]=0; //da nicht alle Zge gepr.

        /* 3.Fall Normalfall */
        else b0[t]=-unendl;


        //evtl. Spiestufe erhhen
        if (t==1 && i[0]==0)
        {
          if (k0*k<=60 && k>0)
          {
            if (alpha0!=beta0) alpha=beta0;
            else
            {
              beta=beta0+2;
              alpha=beta;
            }
          }
          else if (k0*k<=350) beralphabeta(level+1,alpha,beta);
        }

        // Zge nach Bewertung ordnen
        if (sp==startsp) ordnen(k,ind[t],bew[t]);
        //Bewertungen ordnen, wenn Computer an der Reihe

      }
    }

    /*********************** 1 runter *******************/
    else if (az==leererzug || zfi==koenig) //1 down
    {
      

      // schon auf unterster Stufe?
      if (t==0) break;

      // Knig geschlagen => bernehme diese Bewertung als b0
      if (zfi==koenig) b0[t]=bew[t][index]; //Koenig geschlagen

      if (t==alpha+1 && b0[t]<-30) b0[t]=-30; //Bew.unsicher

      // Eins runter
      t--;
      sp=!sp;
      index=ind[t][i[t]];
      bew[t][index]=bew[t][index]-b0[t+1]; //Bewertung vervollst.


      //Patt verhindern erst t==0
      if (pu[!sp]<=7300 && //Mensch hat hchstens noch Knig u.Pferd
         t==0  && //Computer an der Reihe
         bew[t][index]>3500 && //Fr bestimmte Zge Patt oder Matt
         analyse(sp,anztiefen-1)==patt) //Patt
         bew[t][index]=-1000;


      zuruecknehmen(zi[t][index],hi[t]);
      i[t]++; //Nchster Zug an der Reihe

      // Ersetzte b0 durch b[t][index]?
      if (bew[t][index]>b0[t])
      {
        b0[t]=bew[t][index];
        if (t==0) bestzug=zi[0][index];
      }
    }

    /***************** gleichbleibend *****************/
    else //t==beta
    {
      for (l=0;l<k;l++)
        if (bew[t][l]>b0[t]) b0[t]=bew[t][l];
      i[t]=l;
    }
  }
  return bestzug;
}

/****************** SETZELEVEL **************************/
bool jsbrettklasse::setzelevel(int v)
{
	int alpha2,beta2,anztiefen2;
	zug **zi2;
	int **ind2;
	long **bew2;
		
	beralphabeta(v,alpha2,beta2);
	anztiefen2=beta2+4;
	
	//Neuen Speicher allozieren
	
	AllocMatrix(zi2,anztiefen2,mzugm+1);
	AllocMatrix(ind2,anztiefen2,mzugm+1);
	AllocMatrix(bew2,anztiefen2,mzugm+1);

	//Fehler?
	if (zi2==NULL || ind2==NULL || bew2==NULL)
	{
		FreeMatrix(zi2,anztiefen2);
		FreeMatrix(ind2,anztiefen2);
		FreeMatrix(bew2,anztiefen2);
		return false;
	}

	FreeMatrix(zi,anztiefen);
	FreeMatrix(ind,anztiefen);
	FreeMatrix(bew,anztiefen);
	zi=zi2;
	ind=ind2;
	bew=bew2;
	alpha0=alpha=alpha2;
	beta0=beta=beta2;
	anztiefen=anztiefen2;
	level=v;
	return true;
}


/****************** AENDERELEVEL *************************/
/*bool jsbrettklasse::aenderelevel(const int&v)
{

  beralphabeta(v,alpha,beta);
  anztiefen=beta+4;
  level=v;
  alpha0=alpha;
  beta0=beta;

  //Speicher neu anlegen
  FreeMatrix((void**)ind);
  FreeMatrix((void**)bew);
  FreeMatrix((void**)zi);
  zi=(zug**)AllocMatrix(anztiefen,mzugm+1,sizeof(zug));
  ind=(int**)AllocMatrix(anztiefen,mzugm+1,sizeof(int));
  bew=(long**)AllocMatrix(anztiefen,mzugm+1,sizeof(long));

  return (zi && ind && bew);
}*/

/***************** ZIEHEREG ****************/
bool jsbrettklasse::ziehereg(zug& z)
{
  int sp=spvonstein(f[z.x1][z.y1]);

  //cout << z << endl;
  //berlauf?
  if (hz>hzmax) return false;

  //Aktualisiere koenigbew,turmbew fr Rochade
  //Knig schon bewegt?
  if (koenigbew[sp]==-1 && abs(f[z.x1][z.y1])==koenig)
      koenigbew[sp]=hz;

  //Turm schon bewegt?
  if (rturmbew[sp]==-1 && z.x1==7+jan+jan && z.y1==gl[sp])
      rturmbew[sp]=hz;
  if (lturmbew[sp]==-1 && z.x1==0 && z.y1==gl[sp]) lturmbew[sp]=hz;

  //Turm geschlagen?
  if (abs(f[z.x2][z.y2])==turm)
  {
    if (rturmbew[!sp]==-1 &&      //wenn rechter Turm noch nicht bewegt
        z.x2==7+jan+jan &&        //wenn aktueller Zug auf dem Platz des rechten T.endet
        z.y2==gl[!sp]) rturmbew[!sp]=hz;
    else if (lturmbew[!sp]==-1 &&
        z.x2==0 &&
        z.y2==gl[!sp]) lturmbew[!sp]=hz;
   }

  h[hz]=ziehe(z); //Zug auf Brett durchfhren

  //Korrigiere Wert der Spielsteine von Spieler !sp
  pu[!sp]-=punkte[0][abs(h[hz])];

  //Korigiere Wert der Spielsteine von Spieler sp
  if (z.bef) pu[sp]+=punkte[0][abs(z.bef)]-punkte[0][bauer];

  zr[hz]=z; //Zug abspeichern
  hz++; //Nchster Halbzug
  return true;
}

/******************* ZURUECKNEHMENREG ***************/
stein jsbrettklasse::zuruecknehmenreg()
{
  zug z=zr[hz-1];
  int sp=spvonstein(f[z.x2][z.y2]);

  //Unterlauf
  if (hz==0) return 100;

  //Aktualisiere Wert der Steine von Spieler !sp
  pu[!sp]+=punkte[0][abs(h[hz-1])];

  //Aktualisiere Wert der Steine von Spieler sp
  if (z.bef) pu[sp]-=punkte[0][abs(z.bef)]-punkte[0][bauer];

  zuruecknehmen(z,h[hz-1]);
  hz--;

  //fr Rochade
  if (koenigbew[sp]==hz) koenigbew[sp]=-1;
  if (rturmbew[sp]==hz) rturmbew[sp]=-1;
  if (lturmbew[sp]==hz) lturmbew[sp]=-1;

  return h[hz];
}

/******************* moeglichrochade *****************/
bool jsbrettklasse::moeglichrochade(const int& sp,const int& ri)
{
  stein endfeld=(ri==-1)*rochlx+(ri==1)*rochrx;
  int y=gl[sp];
  bool rc=true;
  zug z;

  // Koenig schon bewegt
  if (koenigbew[sp]!=-1) return false;

  // rechter(linker) Turm schon bewegt(oder geschlagen)
  if (ri==1 && rturmbew[sp]!=-1) return false;
  else if (ri==-1 && lturmbew[sp]!=-1) return false;

  // Alle Felder leer von Knig bis Endfeld?
  for (int x=4+ri;x!=endfeld;x+=ri)
    if (f[x][y]!=leer) return false;
  if (f[endfeld][y]!=leer) return false;

  //Schach-berprfung
  z.bef=0;
  z.ep=false;
  z.y1=z.y2=y;
  z.x1=4;
  z.x2=4+ri;
  while (z.x1!=endfeld)
  {
    if (rc==true && schach(sp)) rc=false;
    ziehe(z);
    z.x1+=ri;
    z.x2+=ri;
  }

  //Zurckziehen
  z.x2=4;
  ziehe(z);

  return rc;
}

/*stein jsbrettklasse::feldinhalt(int x, int y)
{
	return f[x][y];
}*/

zug jsbrettklasse::holezug(int i)
{
	if (i<0 || i>hz) return leererzug;
	return zr[i];
}

//********************* operator << fuer brett *******************
ostream& operator<<(ostream& os,const jsbrettklasse& b)
{
  int x,y,i;
  os << b.hz << endl;

  for (i=0;i<b.hz;i++)
    os << b.zr[i] << endl;

  for (x=0;x<=xmax;x++)
  {
    for (y=0;y<=ymax;y++)
    {
      os << b.f[x][y] << '\t';
    } 
    os << '\n';
  }
  return os;
}

//*************** operator >> fuer brett ***********************
istream& operator>>(istream& is,jsbrettklasse& b)
{
  int x,y,i;
 
  is >> b.hz;

  for (i=0;i<b.hz;i++)
  {
    is >> b.zr[i];
  }

  for (x=0;x<=xmax;x++)
    for (y=0;y<=ymax;y++)
      is >> b.f[x][y];
  return is;
}

/******************* SETZEBRETT *********************/
bool jsbrettklasse::setzebrett(stein feld2[xmax+1][ymax+1])
{
	hz=0; //Spielanfang
	koenigx[0]=koenigy[0]=koenigx[1]=koenigy[1]=-1;
	for (int x=0;x<=xmax;x++)
	{
		for (int y=0;y<=ymax;y++)
		{
			f[x][y]=feld2[x][y]; //Feld kopieren
			if (f[x][y]==koenig) //Position des Kaisers setzen
			{
				koenigx[0]=x;
				koenigy[0]=y;
			}
			else if (f[x][y]==-koenig)
			{
				koenigx[1]=x;
				koenigy[1]=y;
			}
		}
	}
	if (koenigx[0]==-1 || koenigx[1]==-1) return false;
	return true;
}


