/* Brett.hh */

#ifndef jsbretth
#define jsbretth

//includes
#include <math.h>
#include <iostream>
#include <vmath/VMath.h>
#include <vgame/boardview.h>

//Defines
#define imfeld(x,y) (0<=x && x<=xmax && 0<=y && y<=ymax)
#define spvonstein(x) (x<0)
//liefert Spieler (0 oder 1) von Stein

//Typdefinitionen
typedef int stein;
typedef struct
{
  int x1;
  int y1;
  int x2;
  int y2;
  stein bef;
  bool ep; //en passant
} zug;
//typedef int bctyp;
typedef enum {normal,istschach,istremis,patt,matt} zctyp;


//Konstanten
const stein leer=0,turm=1,janus=2,pferd=3,laeufer=4,dame=5,koenig=6,
            bauer=7;
const int mzugm=150;
const int xmax=9,ymax=7; //Anzahl Zeilen/Spalten-1
const zug leererzug={0,0,0,0,0,false};
const long punkte[3][8]={{0,500,700,300,300,900,7000,100},
		         {0,480,680,280,280,880,4000,80},
                         {0,5,7,3,3,9,10,8}};
const long unendl=2*punkte[0][koenig];
const int hzmax=500;
const static int dxa[]={0,1,0,-1},dya[]={1,0,-1,0};

using namespace std;


/***************** KLASSE JSBRETTKLASSE *********************/
class jsbrettklasse : public CBoard 
{


//Variablen
public:
  int hz;
  zug zr[hzmax]; //Feld der realen Zge
  stein f[xmax+1][ymax+1]; //Spielfeld
  int rochlx; //Zu welchem Feld geht die Rochade?
  int rochrx;
private:
  int level; //aktuelles Level
  int alpha; //Alpha-Suchtiefe
  int alpha0;
  int beta; //Beta-Suchtiefe
  int beta0;
  int anztiefen; //Maximale Suchtiefe
  long **bew; //Bewertungsfeld
  zug **zi; //Feld der imaginren Zge
  int **ind; //Feld des Sortierungsindexes
  stein h[hzmax]; //Gespeicherte geschlagene Figuren
  int koenigx[2];
  int koenigy[2];
  int jan; //1: Janusschach, 0:normales Schach
  int gl[2]; //Grundlinie der Spieler
  int koenigbew[2]; //in welchem Halbzug wurde Knig bewegt (-1:nicht)
  int rturmbew[2];
  int lturmbew[2];
  int pu[2]; //Wiewiel sind die Steine eines Spielers ing. wert

  //Funktionen
private:

  bool remis();
  /* Diese Funkton gibt wahr zurck, falls eine Remis-Situation
     eingetreten ist*/

  bool allgmoeglich(const zug& z);
  /* Diese Funktion berprft ob ein Zug allgemein mglich ist,
     sie berprft ob
     auf dem Zielfeld sich ein Stein der selben Farbe wie
        auf dem Quellfeld befindet
    */

 bool zuegevonturm
       (const int& t,int& k,zug& z,bool vollesuche,bool schrbew,
        long *b0,int *ix);

 bool zuegevonlaeufer
      (const int& t,int& k,zug& z,bool vollesuche,bool schrbew,
       long *b0,int *ix);

 bool zuegevonpferd
        (const int& t,int& k,zug& z,bool vollesuche,bool schrbew,
         long *b0,int *ix);

 bool zuegevonkoenig
      (const int& t,int& k,zug& z,bool vollesuche,bool schrbew,
       long *b0,int *ix);

 bool zuegevonbauer
      (const int& t,int& k,zug& z,bool vollesuche,bool schrbew,
       long *b0,int *ix,const zug& zv);

 bool zuegevonfeld
        (const int& x,const int& y,const int& t,int &k,long *b0,
         bool schrbew,bool vollesuche,int *ix);
  /* Diese Funktion speichert alle mglichen Zge, die von
     Feld x,y aus mglich sind. Sonstige Parameter:
     t: Position an der die Daten gespeichert werden sollen
     k: Rckgabe: Anzahl Zge
     b0: Optimale Bewertungen, wenn gesetzt wird die Zugtiefe evtl.
         abgebrochen,nicht gesetzt:d.h.=NULL
     schrbew: Wenn wahr schreibt der Computer auch die zugehrigen
              Bewertungen in ein Feld
     vollesuche: wahr: Alle mglichen Zge werden zurckgegeben
                 falsch: Nur bestimmte Zge werden zurckgegeben
*/


  int zuegevonspieler
       (const int& sp,long* b0,bool schrbew,bool vollesuche,
        const int&t,int *ix);
/* Diese Funktion ermittelt alle mglichen Zge von Spieler sp
b0,schrbew,vollesuche,t s.o. Rckgabewert: Anzahl Zugmglichkeiten*/

  long bewerte(const zug& z,const int& t);
/* liefert die Bewertung eines Zuges zurck
2. Paramter Zugtiefe*/

  bool aufnehmen
   (const int&t, int& k,const zug&z,
    const bool& alle,const bool&tubew,long* b0,int *ix);
  /* Speichert einen Zug in ein Feld ab
     t=Tiefe
     k=Anzahl abgespeicherter bis her
     alle=wahr alle Zge werden abgespeichert, sonst nur ausgewhlte
     tubew=wahr: schreibt auch Bewertungen in Feld*/

  bool moeglichfuerturm(const zug& z);

  bool moeglichfuerpferd(const zug& z);

  bool moeglichfuerkoenig(const zug& z);

  bool moeglichfuerbauer(const zug& z);

  bool moeglichfuerlaeufer(const zug& z);

  bool moeglichrochade(const int& sp,const int& ri);

public:

  bool schach(const int&);
  /* Diese Funktion gibt wahr zurck wenn der angegebene Spieler
     sich im Schach befindet, sondt falsch*/

  bool setzebrett(stein feld2[xmax+1][ymax+1]);


    //stein feldinhalt(int x,int y);

  jsbrettklasse();
  //Konstruktor v=Spielstrke

  ~jsbrettklasse();


  void init(int js=1); //inline
  /* Initialisiert das Schachbrett auf Ausgangszustand
     Wird nicht automatisch vom Konstruktor aufgerufen*/

  stein ziehe(zug&); //inline
  /* fhrt angegebenen Zug aus. Gibt Figur zurck, die auf dem
     Zielfeld steht,0 entspricht keine Figur
     Schreibt in Struktur ep=true, wenn en passant-Zug*/

  bool ziehereg(zug& z);
  /* Ziehe angegebenen Zug und speichere ihn
     Von auen sollte nur diese Funktion ausgefhrt werden*/

  void zuruecknehmen(const zug&,const stein&); //inline
  /* Nimmt Zug zurck. Ntige Parameter:
     1. Zug der zurckgenommen werden soll
     2. Stein, die evtl in diesem Zug geschlagen wurde*/

  stein zuruecknehmenreg();
  /* Nimmt gespeicherten Zug zurck und liefert den bei diesem Zug
  gespeicherten Stein zurueck*/

  bool moeglich(zug,const int&); //inline
  /* Gibt zurck, ob ein Zug von einem Spieler mglich ist*/

  bool moeglichvonfigur(const zug&);
  /* Gibt zuck ob ein bestimmter Stein von seiner Bewegungsart
     her diesen Zug ausfhren knnte*/

  zctyp analyse(const int&,int t=0);
  /* analysiert das Spielbrett
     t gibt die Tiefe an, fr Schlagen en passant*/

  zug besterzug(int sp,int& command,int lev);
  /* Der Computer liefert den Zug zurck, den er am besten findet.
     Sp= Spieler am Zug
     abbruch=Flag das indiziert ob der Computer die Suche abbrechen
     soll. Gedacht fr Multithreading*/
  bool setzelevel(int v);

  zug holezug(int hz);

  //bool aenderelevel(const int&v);
  /* Aendert die Spielstrke des Computers
     Parameter: v=neue Spielstrke
                altv=alte Spielstrke
  Sollte diese Operation schiefgehen, wird die alte 
  Spielstrke beibehalten
  */

  void bewegestein(int x1,int y1,int x2,int y2)
  {
    f[x2][y2]=f[x1][y1];
    f[x1][y1]=0;
  }

  int getcontents(int x,int y)
  {
	  return (int)f[x][y];
  }

  friend ostream& operator<<(ostream& os,const jsbrettklasse& brett);
  friend istream& operator>>(istream& is,jsbrettklasse& brett);

};

/******************** ENDE DEFINITION JSBRETTKLASSE ***********/

inline void jummy(){}



//Operationen auf Zug

extern ostream& operator<<(ostream& os,const zug& z);
extern istream& operator>>(istream& is,zug& z);

//*********************** == ******************************
inline bool operator==(const zug& z1,const zug& z2) {
return (z1.x1==z2.x1 && z1.y1==z2.y1 &&
        z1.x2==z2.x2 && z1.y2==z2.y2);
}

//*********************** != ******************************
inline bool operator!=(const zug& z1,const zug& z2) {
return !(z1==z2);
}

/*************** ordnen ************************/
inline void ordnen(const int& k,int* ind,long* bew)
/* k=Anzahl zu ordnende Objekte
   ind*=Indexfeld das geordnet wird
   bew*=Objekte ber die die Ordnung bestimmt wird*/
{
  int h;
  register int i,i2;

  //Index initialisieren
  for (i=0;i<=k;i++)
    ind[i]=i;

  for (i=0;i<k;i++)
    for (i2=i+1;i2<k;i2++)
      if (bew[ind[i]]<bew[ind[i2]])
      {
	h=ind[i];
	ind[i]=ind[i2];
	ind[i2]=h;
      }

}

//********************** ZIEHE *****************************
inline stein jsbrettklasse::ziehe(zug& z){
stein s1=f[z.x1][z.y1],s2=f[z.x2][z.y2];
static stein h;

//en passant
if (abs(f[z.x1][z.y1])==bauer &&
    abs(z.x1-z.x2)==1 &&
    f[z.x2][z.y2]==leer)
{
  z.ep=true;
  bewegestein(z.x1,z.y1,z.x2,z.y2);
  h=f[z.x2][z.y1];
  f[z.x2][z.y1]=leer;
  return h;
}

//Stein bewegen
bewegestein(z.x1,z.y1,z.x2,z.y2);

//Knigsposition speichern
if (abs(s1)==koenig)
{
  koenigx[spvonstein(s1)]=z.x2;
  koenigy[spvonstein(s1)]=z.y2;

  //Rochade
  if (z.x1==4)
  {
    if (z.x2==rochrx)  bewegestein(z.x2+1,z.y1,z.x2-1,z.y1);
    else if (z.x2==rochlx) bewegestein(z.x2-1,z.y1,z.x2+1,z.y1);
  }
}

//Befrderung
if (z.bef!=0) f[z.x2][z.y2]=z.bef;

return s2;
}

//*********************** ZURUECKNEHMEN ***********************
//nimmt Zug z zurck und besetzt evl. vorher geschlagenen Stein
// mit h
inline void jsbrettklasse::zuruecknehmen(const zug& z,const stein& h){
stein s2=f[z.x2][z.y2];

// Figur zurckbewegen
bewegestein(z.x2,z.y2,z.x1,z.y1);

//geschlagene Figur wieder aufs Spielfeld stellen
if (z.ep) f[z.x2][z.y1]=h; //en passant
else f[z.x2][z.y2]=h;


//Knig wurde zurckbewegt?
if (abs(s2)==koenig)
{
  koenigx[spvonstein(s2)]=z.x1;
  koenigy[spvonstein(s2)]=z.y1;

  // Rochade zurcknehmen
  if (z.x1==4)
  {
    if (z.x2==rochrx) bewegestein(z.x2-1,z.y1,z.x2+1,z.y1);
    else if (z.x2==rochlx) bewegestein(z.x2+1,z.y1,z.x2-1,z.y1);
  }
}

//evtl. Befrderung rckgngig machen
if (z.bef!=0) f[z.x1][z.y1]=bauer*sgn(s2);

}

//************************ MOEGLICH ***************************
// gibt zurck ob Zug z von Spieler sp mglich ist
inline bool jsbrettklasse::moeglich(zug z,const int& sp){
/* sp: Spieler fr den Zug berprft werden soll
   z: Zug, der berprft werden soll*/
bool imschach;
stein h;

//jummy();
//steht auf Zielfeld eigener Stein oder auf Quellfeld fremder Stein?
if (!allgmoeglich(z) || spvonstein(f[z.x1][z.y1])!=sp) return false;

h=ziehe(z);
imschach=schach(sp);

zuruecknehmen(z,h);

return (!imschach && moeglichvonfigur(z));
}



//************************* BEWERTE ***************************
inline long jsbrettklasse::bewerte(const zug&z,const int& t) {
long bew,b2=0;
stein h=abs(f[z.x2][z.y2]),st=f[z.x1][z.y1];
int sp=spvonstein(st);
int kx=koenigx[!sp],ky=koenigy[!sp],ak;


//Materialstrategie
if (t>alpha+2) ak=2;
else if (t>alpha) ak=1;
else ak=0;
if (z.ep) bew=punkte[ak][bauer];
else bew=punkte[ak][h];
//Befrderung
if (z.bef) bew+=punkte[ak][abs(z.bef)]-punkte[ak][bauer];

// Verhindern, da Computer falsch auf Schach reagiert
if (h==koenig && t==1) bew=bew+bew+bew;

// Angriffsstrategie
if (abs(st)!=koenig || pu[!sp]<=7100 && pu[sp]>=7500)
    bew=bew+10-abs(z.x2-kx)+abs(z.x1-kx)
              -abs(z.y2-ky)+abs(z.y1-ky);

//Einzelnen Knig nicht in die Ecke drngen lassen
if (pu[sp]<=7100 && abs(st)==koenig)
   bew=bew-20*(abs(z.x2-4)-abs(z.y2-4)+4)+10;

//Rochade untersttzen
if ((abs(st)==koenig || abs(st)==turm) && hz<10) bew-=2;

if (bew<0) return 0;
return bew;
}

/*
if (spvonstein(f[z.x1][z.y1])==0 && z.y2!=ymax) b2=z.y2-z.y1+10;
else if (spvonstein(f[z.x1][z.y1])==1 && z.y2!=0) b2=z.y1-z.y2+10;
if (bew+b2<0) return 0;
return bew+b2;
}
*/

/************************ ZUG AUFNEHMEN ***********************/
inline bool jsbrettklasse::aufnehmen
(const int& t,int& k,const zug& z,const bool& alle,const bool& tubew,
 long* b0,int *ix)
/* t=Tiefe
   k=Anzahl aufgenomener Zge
   alle=alle aufnehmen
   tubew=Bewertungen auch abspeichern
   Gibt bei Zugoptimierung falsch zurueck
*/
{


  //Nur Aufnahme, wenn Figuren geschlagen werden?
  if (!alle && f[z.x2][z.y2]==0) return true;

  zi[t][k]=z;
  if (tubew) bew[t][k]=bewerte(z,t);


  /*Mensch-Optimierung
    Der Zug, den der Mensch im letzten Zug macht ist so schlecht
    fr den Computer, da er auf keinen Fall den dafr
    zugrundeliegenden Zug ausfhren wird*/

  if ((t==beta || t==alpha && alpha>1) && b0)
  {
    if (bew[t-1][ind[t-1][ix[t-1]]]-bew[t][k]<b0[t-1])
    {
      if (t==beta || (t==alpha && b0[t-1]>0)) return false;
    }
  }

  k++;
  return true;
}

#define zugaufnehmen if (!aufnehmen(t,k,z,vollesuche,schrbew,b0,ix)) \
                     return false;

/********************** ZUEGEVONTURM ************************/
inline bool jsbrettklasse::zuegevonturm
  (const int& t,int& k,zug& z,bool vollesuche,bool schrbew,
   long *b0,int *ix)
/* Rckgabewert: true: alle Zge wurden berechnet
                 false: es wurde eine Mensch-Optimierung durchg.,
                 es brauchten nicht alle Zge berechnet werden*/
{
  for (int i=0;i<=3 ;i++ )
  {
    z.x2=z.x1;
    z.y2=z.y1;
    do
    {
      z.x2=z.x2+dxa[i];
      z.y2=z.y2+dya[i];
      if (!imfeld(z.x2,z.y2)) break;
      if (allgmoeglich(z)) zugaufnehmen;
    } while (f[z.x2][z.y2]==leer  ); /* enddo */
  } /* endfor */
  return true;
}

/********************* ZUEGEVONLAEUFER *******************/
inline bool jsbrettklasse::zuegevonlaeufer
 (const int& t,int& k,zug& z,bool vollesuche,bool schrbew,
   long *b0,int *ix)
{
  int dx,dy;

  for (dx=-1;dx<=1;dx=dx+2) //Bewegungsrichtung
  {
    for (dy=-1;dy<=1;dy=dy+2)
    {
      z.x2=z.x1; //Zurcksetzen
      z.y2=z.y1;
      do
      {
        /* Gehe solange in diese Richtung bis auerhalb des
           Feldes oder auf andere Figur getroffen */
        z.x2=z.x2+dx;
        z.y2=z.y2+dy;
        if (!imfeld(z.x2,z.y2)) break;
        if (allgmoeglich(z)) zugaufnehmen(z);
      } while (f[z.x2][z.y2]==leer  ); /* enddo */
    }
  }
  return true;
}

/********************* ZUEGEVONPFERD **********************/
inline bool jsbrettklasse::zuegevonpferd
 (const int& t,int& k,zug& z,bool vollesuche,bool schrbew,
   long *b0,int *ix)
{
int dx,dy,i;

  for (dx=-2;dx<=2 ;dx++ )
  {
    if (dx==0) continue;
    dy=3-abs(dx);
    for (i=1;i<=2 ;i++,dy=-dy )
    {
      z.x2=z.x1+dx; //Alle Pferdpositionen
      z.y2=z.y1+dy;
      if (imfeld(z.x2,z.y2) && allgmoeglich(z)) zugaufnehmen;
    } /* endfor */
  } /* endfor */
  return true;
}

/*************************** ZUEGEVONKOENIG *********************/
inline bool jsbrettklasse::zuegevonkoenig
 (const int& t,int& k,zug& z,bool vollesuche,bool schrbew,
   long *b0,int *ix)
{

  for   (z.x2=z.x1-1;z.x2<=z.x1+1;z.x2++)
  {
    for (z.y2=z.y1-1;z.y2<=z.y1+1;z.y2++)
    {
      if (imfeld(z.x2,z.y2) && (z.x2!=z.x1 || z.y2!=z.y1) &&
          allgmoeglich(z)) zugaufnehmen;
    }
  }
  return true;
}

/*********************** ZUEGEVONBAUER ***********************/
inline bool jsbrettklasse::zuegevonbauer
 (const int& t,int& k,zug& z,bool vollesuche,bool schrbew,
   long *b0,int *ix,const zug& zv=leererzug)
{
  stein qf=f[z.x1][z.y1];
  int dy=sgn(qf),sp=spvonstein(qf);

  z.x2=z.x1; //Eins nach vorne
  z.y2=z.y1+dy;
  if (f[z.x2][z.y2]==leer && imfeld(z.x2,z.y2))
  {
    if (z.y2==ymax || z.y2==0) //Befrderung
    {
      z.bef=dame*dy;
      zugaufnehmen;
      z.bef=janus*dy;
      zugaufnehmen;
      z.bef=0;
    }
    else zugaufnehmen;

    z.y2=z.y2+dy; //Zwei nach vorne
    if (z.y1==gl[sp]+dy && f[z.x2][z.y2]==leer) zugaufnehmen;
  }

  //1 Feld diagonal
  for (z.x2=z.x1-1;z.x2<=z.x1+1;z.x2+=2) //Ein Feld quer
  {
    z.y2=z.y1+dy; //Ein Feld gerade
    if (imfeld(z.x2,z.y2))
    {
      if (f[z.x2][z.y2]!=leer && spvonstein(f[z.x2][z.y2])!=sp)
      {
        if (z.y2==ymax || z.y2==0) //Befrderung
        {
          z.bef=dame*dy;
          zugaufnehmen;
          z.bef=janus*dy;
          zugaufnehmen;
          z.bef=0;
        } else  zugaufnehmen;
      }
      else if (abs(f[z.x2][z.y1])==bauer &&
              zv!=leererzug &&
              zv.y2==z.y1 &&
              zv.x2==z.x2 &&
              abs(zv.y1-zv.y2)==2)
      {
        z.ep=true;
        zugaufnehmen;
        z.ep=false;
      }
    }
  }
  return true;
}



//************************** ZUEGEVONFELD **********************
inline bool jsbrettklasse::zuegevonfeld
(const int&x,const int&y,const int& t,int& k,long *b0,
bool schrbew,bool vollesuche,int *ix)
/* Rckgabewert: true: Alle Zge wurden berechnet
                 false: Nicht alle Zge brauchten berechnet
                        zu werden, da eine Mensch-Optimierung
                        durchgefhrt wurde*/
{
zug z={x,y,0,0,0},zv;
bool b;
const static int dxa[]={0,1,0,-1},dya[]={1,0,-1,0};

switch (abs(f[x][y])) {
case koenig:
  return zuegevonkoenig(t,k,z,vollesuche,schrbew,b0,ix);
case laeufer:
  return zuegevonlaeufer(t,k,z,vollesuche,schrbew,b0,ix);
case turm:
  return zuegevonturm(t,k,z,vollesuche,schrbew,b0,ix);
case pferd:
  return zuegevonpferd(t,k,z,vollesuche,schrbew,b0,ix);
case janus:
  b=zuegevonpferd(t,k,z,vollesuche,schrbew,b0,ix);
  if (!b) return false;
  return zuegevonlaeufer(t,k,z,vollesuche,schrbew,b0,ix);
case dame:
  b=zuegevonturm(t,k,z,vollesuche,schrbew,b0,ix);
  if (!b) return false;
  return zuegevonlaeufer(t,k,z,vollesuche,schrbew,b0,ix);
case bauer:
  if (t>0 && t<=anztiefen-2) zv=zi[t-1][ind[t-1][ix[t-1]]];
  else if (t==0 && hz!=0) zv=zr[hz-1];
  else zv=leererzug;
  return zuegevonbauer(t,k,z,vollesuche,schrbew,b0,ix,zv);
} /* endswitch */
return true;
}
#endif





























